Package: clusteredMSM
Type: Package
Title: Nonparametric Analysis of Clustered Multistate Processes
Version: 0.1.0
Authors@R: 
    person(given = "Giorgos",
           family = "Bakoyannis",
           role = c("aut", "cre"),
           email = "gbakogia@iu.edu",
           comment = c(ORCID = "0000-0002-2789-2497"))
Description: Nonparametric estimation of population-averaged transition
    probabilities, with cluster-bootstrap pointwise confidence intervals,
    simultaneous confidence bands, and two-sample Kolmogorov-Smirnov-type
    tests for clustered or independent multistate process data.
    Estimation follows Bakoyannis (2021) <doi:10.1111/biom.13327>;
    two-sample inference for the cluster-randomized and
    independent-samples designs follows Bakoyannis and Bandyopadhyay
    (2022) <doi:10.1007/s10463-021-00819-x>. Both methods use the
    working-independence Aalen-Johansen estimator. The package supports
    both progressive (acyclic) and non-monotone (e.g., illness-death
    with recovery) multistate processes, right censoring, left
    truncation, and informative cluster size. The user supplies data
    in interval format (one row per mutually-exclusive time interval
    per subject) and interacts with the package through a single
    formula-based function, patp().
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 3.5.0)
Imports: survival, stats, utils
Suggests: mstate, testthat (>= 3.0.0), knitr, rmarkdown
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/gbakoyannis/clusteredMSM
BugReports: https://github.com/gbakoyannis/clusteredMSM/issues
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-05-20 02:07:01 UTC; giorgos
Author: Giorgos Bakoyannis [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2789-2497>)
Maintainer: Giorgos Bakoyannis <gbakogia@iu.edu>
Repository: CRAN
Date/Publication: 2026-05-27 19:20:08 UTC
Built: R 4.5.2; ; 2026-05-27 21:25:00 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(print,msm)
S3method(print,patp)
S3method(print,summary.patp)
S3method(summary,patp)
export(ci_cloglog)
export(cluster_boot)
export(confidence_band)
export(cut_at_lm)
export(fit_chaz)
export(intervals_to_long)
export(ks_pvalue)
export(msm)
export(patp)
export(prodint_AJ)
export(state_at)
export(trans_mat)
export(validate_intervals)
importFrom(survival,Surv)
importFrom(survival,basehaz)
importFrom(survival,coxph)
importFrom(survival,strata)

# clusteredMSM

Nonparametric analysis of clustered multistate process data.

`clusteredMSM` provides population-averaged transition probability
estimates, pointwise confidence intervals, simultaneous confidence
bands, and two-sample Kolmogorov-Smirnov-type tests for multistate
process data with cluster-correlated observations. Estimation follows
[Bakoyannis (2021)](https://doi.org/10.1111/biom.13327); two-sample
inference for the cluster-randomized and independent-samples designs
follows [Bakoyannis & Bandyopadhyay
(2022)](https://doi.org/10.1007/s10463-021-00819-x). Both rest on the
working-independence Aalen-Johansen estimator with a cluster-bootstrap
variance.

Unlike its predecessor (the `clustered-multistate` repository, which
relied on the `mstate` package), `clusteredMSM` is self-contained
(depending only on `survival`) and supports **non-monotone multistate
processes**, including illness-death with recovery and other models with
cyclic transitions.

## Installation

``` r
# install.packages("devtools")
devtools::install_github("gbakoyannis/clusteredMSM")
```

After CRAN release:

``` r
install.packages("clusteredMSM")
```

## A single function with a formula interface

`clusteredMSM` exposes one main function, `patp()`, modelled after
`survival::Surv()`:

``` r
library(clusteredMSM)

# Synthetic clustered illness-death-with-recovery data (40 subjects,
# 8 clusters); see ?example_msm.
data(example_msm)

# Define the transition structure (illness-death with recovery)
tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
                  names = c("Healthy", "Ill", "Dead"))

# One-sample analysis: P(Ill at t | Healthy at 0)
fit <- patp(msm(Tstart, Tstop, Sstart, Sstop) ~ 1,
            data = example_msm, tmat = tmat,
            id = "id", cluster = "cluster",
            h = 1, j = 2, s = 0,
            B = 1000, cband = TRUE)
fit
```

If the formula's right-hand side has a grouping variable, `patp()`
automatically estimates both group-specific curves AND tests their
equality:

``` r
# Two-sample analysis (estimate + test in one call)
tt <- patp(msm(Tstart, Tstop, Sstart, Sstop) ~ treatment,
           data = example_msm, tmat = tmat,
           id = "id", cluster = "cluster",
           h = 1, j = 2, B = 1000)
tt
```

### Loading your own data

The same example is shipped as a CSV under `inst/extdata/`, so you can
mimic the typical workflow of reading a user-supplied file:

``` r
f <- system.file("extdata", "example_data.csv", package = "clusteredMSM")
mydata <- read.csv(f)
head(mydata)
```

## Input data format

Each row of your data represents one **mutually-exclusive time
interval** for one subject, with columns:

| Column    | Description                                                 |
|-----------|-------------------------------------------------------------|
| `Tstart`  | Numeric start time of the interval                          |
| `Tstop`   | Numeric end time of the interval                            |
| `Sstart`  | Integer state occupied during the interval                  |
| `Sstop`   | Integer state at `Tstop` (or equal to `Sstart` if censored) |
| `id`      | Subject identifier                                          |
| `cluster` | (optional) cluster identifier                               |
| (group)   | (optional) binary grouping variable                         |

The column names are arbitrary — `msm(...)` and the `id`/`cluster`
arguments tell the package which is which.

**Censoring** is encoded as `Sstart == Sstop` on the **final row** of a
subject's record. Subjects in absorbing states have no row after them.

Within each subject, intervals must be: - **Temporally contiguous:**
`Tstop[k] == Tstart[k+1]` - **State contiguous:**
`Sstop[k] == Sstart[k+1]`

Validation is strict and informative — any violation triggers an error
with a clear message.

## Examples of valid input

**Progressive illness-death (subject who got ill, then died):**

| id  | Tstart | Tstop | Sstart | Sstop |
|-----|--------|-------|--------|-------|
| 1   | 0.0    | 1.5   | 1      | 2     |
| 1   | 1.5    | 3.0   | 2      | 3     |

**Subject censored healthy:**

| id  | Tstart | Tstop | Sstart | Sstop |
|-----|--------|-------|--------|-------|
| 2   | 0.0    | 4.0   | 1      | 1     |

**Recovery (Healthy → Ill → Healthy → censored):**

| id  | Tstart | Tstop | Sstart | Sstop |
|-----|--------|-------|--------|-------|
| 3   | 0.0    | 1.0   | 1      | 2     |
| 3   | 1.0    | 2.0   | 2      | 1     |
| 3   | 2.0    | 3.5   | 1      | 1     |

## Core functions

| Function | Purpose |
|----|----|
| `patp()` | The main user-facing function — formula-based estimation and testing. |
| `msm()` | Constructor for multistate intervals; used inside the formula. |
| `trans_mat()` | Build a K x K transition matrix. |
| `validate_intervals()` | Validate user data (called automatically by `patp()`; usable directly). |

## References

> Bakoyannis, G. (2021). Nonparametric analysis of nonhomogeneous
> multistate processes with clustered observations. *Biometrics*, 77(2),
> 533-546. <doi:10.1111/biom.13327>

> Bakoyannis, G., & Bandyopadhyay, D. (2022). Nonparametric tests for
> multistate processes with clustered data. *Annals of the Institute of
> Statistical Mathematics*, 74(5), 837-867.
> <doi:10.1007/s10463-021-00819-x>

You can retrieve the BibTeX entries within R via
`toBibtex(citation("clusteredMSM"))`.

## License

GPL-3

#' Pointwise Confidence Intervals via Complementary Log-Log Transformation
#'
#' Computes pointwise confidence intervals for a transition probability
#' estimate using the cloglog transformation \eqn{g(p) = \log(-\log p)}
#' with a delta-method standard error scaling. The bootstrap is run on the 
#' original probability scale; the cloglog SE is then derived analytically.
#'
#' @param point Numeric vector of point estimates in (0, 1), length T.
#' @param se Numeric vector of bootstrap standard errors of \code{point}
#'   on the probability scale, same length as \code{point} (i.e.
#'   \code{apply(boot_matrix, 1, sd, na.rm = TRUE)}).
#' @param level Confidence level. Default 0.95.
#'
#' @return A list with elements \code{ll} and \code{ul}: numeric vectors
#'   of lower and upper confidence limits, the same length as
#'   \code{point}.
#'
#' @details
#' By the delta method,
#' \eqn{\mathrm{SE}(g(\hat P)) = \mathrm{SE}(\hat P) / |\hat P \log \hat P|}
#' (here \eqn{|g'(p)| = 1/|p \log p|} on (0,1)). The interval is built
#' symmetrically on the cloglog scale and back-transformed via
#' \eqn{p = \exp(-\exp(\cdot))}. Because \eqn{g} is monotone
#' decreasing on (0, 1), the upper end on the cloglog scale becomes
#' the \emph{lower} end on the probability scale, and vice versa.
#'
#' For point estimates exactly equal to 0 or 1 the cloglog is
#' undefined and \code{NA} is returned at those positions.
#'
#' @export
ci_cloglog <- function(point, se, level = 0.95) {

  if (length(point) != length(se)) {
    stop("'point' and 'se' must have the same length")
  }

  z <- stats::qnorm(1 - (1 - level) / 2)

  ll <- ul <- rep(NA_real_, length(point))
  ok <- !is.na(point) & point > 0 & point < 1 &
        !is.na(se)   & is.finite(se)

  if (!any(ok)) return(list(ll = ll, ul = ul))

  P    <- point[ok]
  g_P  <- log(-log(P))
  se_g <- se[ok] / abs(P * log(P))   # delta-method scaling, |g'(p)| = 1/|p log p|

  # g is decreasing on (0, 1):
  # upper end on cloglog scale -> lower end on probability scale.
  ll[ok] <- exp(-exp(g_P + z * se_g))
  ul[ok] <- exp(-exp(g_P - z * se_g))

  list(ll = ll, ul = ul)
}


#' Simultaneous Confidence Band via Cluster-Bootstrap Quantile
#'
#' Constructs a simultaneous (uniform-over-time) confidence band for a
#' transition probability curve. The band is built on the cloglog
#' scale: for each bootstrap replicate, the standardized supremum
#' \eqn{\sup_t |g(\hat P_b(t)) - g(\hat P(t))| / \mathrm{SE}_g(t)} is
#' computed, where \eqn{\mathrm{SE}_g(t)} is the delta-method SE on
#' the cloglog scale. The \code{level} quantile of those suprema is
#' the critical value, and the band is back-transformed to the
#' probability scale.
#'
#' @param point Numeric vector of point estimates over a time grid.
#' @param boot Numeric matrix of bootstrap replicates of \code{point}
#'   on the original probability scale: rows correspond to time
#'   points (matching \code{point}), columns to bootstrap replicates.
#' @param times Numeric vector of times matching the rows of \code{boot}
#'   and the entries of \code{point}.
#' @param level Confidence level. Default 0.95.
#' @param trim Numeric vector of length 2. Lower and upper quantiles of
#'   the jump-time distribution at which to clip the band (avoids the
#'   "fans out" behavior near the extremes of follow-up). Default
#'   \code{c(0.05, 0.95)}.
#'
#' @return A list with elements \code{ll.band} and \code{ul.band}:
#'   numeric vectors of band limits, with \code{NA} outside the trimmed
#'   range or where \code{point} is at the boundary 0/1.
#'
#' @details
#' Only one bootstrap on the original (probability) scale is required:
#' \eqn{\mathrm{SE}_g(t)} is obtained by the delta method from
#' \eqn{\mathrm{SE}(\hat P(t)) = \mathrm{sd}(\hat P^*(t))}, and the
#' cloglog values \eqn{g(\hat P_b(t))} are computed by transforming
#' the existing replicates pointwise -- no separate cloglog-scale
#' bootstrap is run.
#'
#' @section Note:
#' This version uses an equal-precision-type band on the cloglog
#' scale, in which the bootstrap deviation is standardized pointwise
#' by the delta-method standard error of \eqn{P(t)}. By the self-
#' cancellation of \eqn{\sqrt{n}} factors, this is asymptotically
#' equivalent to standardizing the limit process by its asymptotic
#' standard error on the cloglog scale, which is the standard
#' equal-precision construction. This construction is asymptotically
#' valid under conditions C1-C6 of Bakoyannis (2021), but differs
#' from the Hall-Wellner-type weight proposed in Section 2.3 of that
#' paper. Both constructions yield asymptotically valid simultaneous
#' bands. The paper's Hall-Wellner-type construction will be added as
#' a \code{band_method} option in a future release and will become
#' the default.
#'
#' @references
#' Bakoyannis, G. (2021). Nonparametric analysis of nonhomogeneous
#' multistate processes with clustered observations.
#' \emph{Biometrics}, 77(2), 533-546. \doi{10.1111/biom.13327}
#'
#' @export
confidence_band <- function(point, boot, times,
                            level = 0.95, trim = c(0.05, 0.95)) {

  if (!is.matrix(boot)) {
    stop("'boot' must be a numeric matrix (rows = time points, cols = replicates)")
  }
  if (nrow(boot) != length(point)) {
    stop("nrow(boot) must equal length(point)")
  }
  if (length(times) != length(point)) {
    stop("'times' must have the same length as 'point'")
  }

  na_band <- function() list(ll.band = rep(NA_real_, length(point)),
                             ul.band = rep(NA_real_, length(point)))

  jump_times <- times[c(TRUE, diff(point) != 0)]
  if (length(jump_times) == 0L) return(na_band())
  qs <- stats::quantile(jump_times, probs = trim)

  in_range <- !is.na(point) & point > 0 & point < 1 &
              times >= qs[1] & times <= qs[2]
  if (!any(in_range)) return(na_band())

  P    <- point[in_range]
  B_in <- boot[in_range, , drop = FALSE]

  # Probability-scale SE of P_hat, then delta-method cloglog SE.
  se   <- apply(B_in, 1L, stats::sd, na.rm = TRUE)
  se_g <- se / abs(P * log(P))

  # Cloglog of P_hat and of replicates; mask invalid replicate values.
  g_P     <- log(-log(P))
  invalid <- !is.finite(B_in) | B_in <= 0 | B_in >= 1
  G       <- B_in
  G[invalid] <- NA_real_
  G       <- log(-log(G))

  # Standardized cloglog deviations; replicate-by-replicate sup
  resid <- abs(sweep(G, 1L, g_P, FUN = "-")) / se_g
  sups  <- apply(resid, 2L, max, na.rm = TRUE)
  sups  <- sups[is.finite(sups)]
  if (length(sups) == 0L) return(na_band())
  c_a <- stats::quantile(sups, probs = level, na.rm = TRUE)

  ll.band <- ul.band <- rep(NA_real_, length(point))
  # g is decreasing => upper-cloglog = lower-probability
  ll.band[in_range] <- exp(-exp(g_P + c_a * se_g))
  ul.band[in_range] <- exp(-exp(g_P - c_a * se_g))

  list(ll.band = ll.band, ul.band = ul.band)
}


#' Two-Sample Kolmogorov-Smirnov-Type Test via Cluster Bootstrap
#'
#' Computes a p-value for the null hypothesis that a transition
#' probability curve is equal across two groups, based on the supremum
#' of the absolute difference between group-specific Aalen-Johansen
#' estimators and a cluster-bootstrap reference distribution.
#'
#' @param diff_point Numeric vector. Observed group difference of the
#'   point estimator (\eqn{\hat P_1 - \hat P_0}) on a fixed time grid.
#' @param diff_boot Numeric matrix. Cluster bootstrap replicates of the
#'   group difference: rows = time points (matching \code{diff_point}),
#'   columns = replicates.
#' @param scale Numeric scalar. Asymptotic scaling factor applied to
#'   both the observed sup-statistic and the bootstrap analogue.
#'   Use \eqn{\sqrt{n}} (\eqn{n} = total clusters) for the
#'   dependent-groups design (Bakoyannis 2021, Theorem 3); use
#'   \eqn{\sqrt{n_1 n_2 / (n_1 + n_2)}} for the cluster-randomized
#'   design (Bakoyannis & Bandyopadhyay 2022, Theorem 2).
#'
#' @return A list with elements \code{statistic} (the observed
#'   sup-statistic, on the chosen scale) and \code{p.value}.
#'
#' @details
#' The test statistic is \eqn{T = c_n \sup_t | \hat P_1(t) -
#' \hat P_0(t) |} with \eqn{c_n =} \code{scale}, and the null
#' distribution is approximated by the bootstrap analogue applied to
#' centered bootstrap differences. The empirical p-value is invariant
#' to \code{scale} (it appears on both sides of the comparison); the
#' role of \code{scale} is to put the reported statistic on the
#' correct asymptotic scale per the relevant theorem.
#'
#' This v0.1 implementation uses a unit weight: every time point on
#' the grid contributes equally to the supremum. Bakoyannis (2021)
#' Section 2.5 instead recommends a time-varying weight
#' \eqn{W(t) = \prod_p Y_p(t) / \sum_p Y_p(t)} (the harmonic mean of
#' the per-group at-risk processes), which downweights regions where
#' one group's at-risk set is small and naturally tames tail
#' instability of the difference estimator. A weighted variant
#' following Bakoyannis (2021) Section 2.5 and Bakoyannis & Bandyopadhyay
#' (2022) is planned for v0.2.
#'
#' @export
ks_pvalue <- function(diff_point, diff_boot, scale) {

  if (nrow(diff_boot) != length(diff_point)) {
    stop("nrow(diff_boot) must equal length(diff_point)")
  }

  T_obs <- scale * max(abs(diff_point), na.rm = TRUE)

  # Bootstrap analogue: center each bootstrap diff around the observed
  # diff (Bakoyannis & Bandyopadhyay 2022, Section 3).
  centered <- sweep(diff_boot, 1, diff_point, FUN = "-")
  T_boot   <- scale * apply(abs(centered), 2, max, na.rm = TRUE)

  list(statistic = T_obs,
       p.value   = mean(T_boot >= T_obs, na.rm = TRUE))
}

test_that("ci_cloglog returns bounds straddling the point estimate", {
  point <- c(0.2, 0.5, 0.8)
  se    <- c(0.05, 0.05, 0.05)
  ci <- ci_cloglog(point, se)

  expect_named(ci, c("ll", "ul"))
  expect_true(all(ci$ll < point))
  expect_true(all(ci$ul > point))
  expect_true(all(ci$ll > 0 & ci$ul < 1))
})

test_that("ci_cloglog matches hand-computed delta-method interval", {
  ## P_hat = 0.1, SE on probability scale = 0.02:
  ##   g(0.1)        ~= 0.8340
  ##   |g'(0.1)|     = 1/(0.1 * |log 0.1|) ~= 4.3429
  ##   se_g          ~= 4.3429 * 0.02 ~= 0.0869
  ##   95% CI(cloglog) = 0.8340 +/- 1.96 * 0.0869
  ##   Back-transform  -> (0.0652, 0.1434)
  ci <- ci_cloglog(point = 0.1, se = 0.02, level = 0.95)
  expect_equal(ci$ll, 0.0652, tolerance = 1e-3)
  expect_equal(ci$ul, 0.1434, tolerance = 1e-3)
  expect_true(ci$ll < 0.1 && 0.1 < ci$ul)
  expect_true(ci$ll > 0  && ci$ul < 1)
})

test_that("ci_cloglog handles boundary point estimates with NA", {
  point <- c(0, 0.5, 1)
  se    <- c(0.1, 0.1, 0.1)
  ci <- ci_cloglog(point, se)

  expect_true(is.na(ci$ll[1]))
  expect_true(is.na(ci$ul[1]))
  expect_false(is.na(ci$ll[2]))
  expect_true(is.na(ci$ll[3]))
})

test_that("ci_cloglog narrows with smaller SE", {
  point <- rep(0.5, 3)
  ci_wide   <- ci_cloglog(point, rep(0.10, 3))
  ci_narrow <- ci_cloglog(point, rep(0.01, 3))

  expect_true(all(ci_narrow$ul - ci_narrow$ll <
                  ci_wide$ul   - ci_wide$ll))
})

test_that("ci_cloglog level argument widens the interval", {
  point <- 0.5
  se    <- 0.05
  ci_95 <- ci_cloglog(point, se, level = 0.95)
  ci_99 <- ci_cloglog(point, se, level = 0.99)

  expect_true(ci_99$ul - ci_99$ll > ci_95$ul - ci_95$ll)
})

test_that("ci_cloglog: doubling SE roughly doubles cloglog-scale width", {
  point <- 0.3
  ci1 <- ci_cloglog(point, se = 0.02)
  ci2 <- ci_cloglog(point, se = 0.04)

  g       <- function(p) log(-log(p))
  width_g <- function(ci) g(ci$ll) - g(ci$ul)  # g is decreasing
  expect_equal(width_g(ci2) / width_g(ci1), 2, tolerance = 1e-12)
})

test_that("ci_cloglog: bounds in (0,1) and bracket the point", {
  set.seed(7)
  point <- stats::runif(20, 0.05, 0.95)
  se    <- rep(0.05, 20)
  ci    <- ci_cloglog(point, se)

  expect_true(all(ci$ll > 0 & ci$ll < point))
  expect_true(all(ci$ul > point & ci$ul < 1))
})

test_that("ci_cloglog validates input lengths", {
  expect_error(ci_cloglog(c(0.3, 0.5), c(0.1)), "same length")
})


## Helper for confidence_band tests: T x B replicate matrix.
make_boot <- function(point, se, B = 500, seed = 1) {
  set.seed(seed)
  T_ <- length(point)
  if (length(se) == 1L) se <- rep(se, T_)
  matrix(stats::rnorm(T_ * B,
                      mean = rep(point, B),
                      sd   = rep(se,    B)),
         nrow = T_, ncol = B)
}

test_that("confidence_band returns NA outside trim range and valid inside", {
  set.seed(1)
  times <- seq(0.1, 1.0, length.out = 20)
  point <- pmax(0.01, pmin(0.99, sort(stats::runif(20, 0.1, 0.9))))
  boot  <- make_boot(point, 0.05, B = 200, seed = 13)

  band <- confidence_band(point, boot, times = times)
  expect_named(band, c("ll.band", "ul.band"))

  expect_true(any(!is.na(band$ll.band)))
  ok <- !is.na(band$ll.band)
  expect_true(all(band$ll.band[ok] <= point[ok] + 1e-8))
  expect_true(all(band$ul.band[ok] >= point[ok] - 1e-8))
})

test_that("confidence_band is at least as wide as pointwise CI", {
  set.seed(2)
  times <- seq(0.1, 1.0, length.out = 30)
  point <- pmax(0.05, pmin(0.95, sort(stats::runif(30, 0.1, 0.9))))
  boot  <- make_boot(point, 0.05, B = 500, seed = 5)
  se    <- apply(boot, 1L, stats::sd, na.rm = TRUE)

  ci   <- ci_cloglog(point, se)
  band <- confidence_band(point, boot, times = times,
                          trim = c(0, 1))

  ok <- !is.na(band$ll.band) & !is.na(ci$ll)
  expect_true(all(band$ll.band[ok] <= ci$ll[ok] + 1e-8))
  expect_true(all(band$ul.band[ok] >= ci$ul[ok] - 1e-8))
})

test_that("confidence_band validates dimensions", {
  expect_error(confidence_band(c(0.3, 0.5),
                               matrix(0, 3, 100),
                               times = c(1, 2)),
               "nrow\\(boot\\)")
})


test_that("ks_pvalue is in [0, 1] and reflects observed statistic", {
  set.seed(2)
  diff_point <- stats::rnorm(50, 0, 0.05)
  diff_boot  <- matrix(stats::rnorm(50 * 500, 0, 0.05), 50, 500)

  ks <- ks_pvalue(diff_point, diff_boot, scale = sqrt(100))
  expect_named(ks, c("statistic", "p.value"))
  expect_true(ks$p.value >= 0 && ks$p.value <= 1)
  expect_true(ks$statistic >= 0)
})

test_that("ks_pvalue rejects under large observed difference", {
  set.seed(3)
  # Big observed difference, small bootstrap noise. Centered-bootstrap
  # convention: diff_boot replicates the observed statistic, so its
  # mean tracks diff_point (0.5), not 0.
  diff_point <- rep(0.5, 30)
  diff_boot  <- matrix(stats::rnorm(30 * 200, 0.5, 0.01), 30, 200)

  ks <- ks_pvalue(diff_point, diff_boot, scale = sqrt(100))
  expect_lt(ks$p.value, 0.05)
})

test_that("ks_pvalue does not reject under small observed difference", {
  set.seed(4)
  # Small observed difference, larger bootstrap variability
  diff_point <- stats::rnorm(30, 0, 0.01)
  diff_boot  <- matrix(stats::rnorm(30 * 500, 0, 0.1), 30, 500)

  ks <- ks_pvalue(diff_point, diff_boot, scale = sqrt(100))
  expect_gt(ks$p.value, 0.05)
})

#' Build a Transition Matrix for a Multistate Process
#'
#' Constructs a square transition matrix in the format expected by
#' \code{\link{patp}} and \code{\link{prodint_AJ}}. It supports cyclic transitions
#' (e.g., illness-death with recovery).
#'
#' @param x A list of length K. Element \code{k} is an integer vector of
#'   states reachable directly from state \code{k}. Use
#'   \code{integer(0)} or \code{NULL} for absorbing states.
#' @param names Optional character vector of length K with state names.
#'   If \code{NULL}, states are named \code{"1"}, \code{"2"}, ..., \code{"K"}.
#'
#' @return A K x K integer matrix. Entry \code{[h, j]} is the integer
#'   transition ID for the direct transition h -> j, or \code{NA} if no
#'   such transition exists. Transition IDs are assigned in row-major
#'   order (all transitions out of state 1 first, then state 2, etc.).
#'   The matrix has \code{dimnames} \code{list(from = names, to = names)}.
#'
#' @examples
#' # Illness-death without recovery (progressive)
#' trans_mat(list(c(2, 3), 3, integer(0)),
#'           names = c("Healthy", "Ill", "Dead"))
#'
#' # Illness-death WITH recovery (non-monotone)
#' trans_mat(list(c(2, 3), c(1, 3), integer(0)),
#'           names = c("Healthy", "Ill", "Dead"))
#'
#' # Four-state multistate model with recovery from two intermediate states
#' trans_mat(list(c(2, 3, 4), c(1, 4), c(1, 4), integer(0)))
#'
#' @export
trans_mat <- function(x, names = NULL) {

  if (!is.list(x)) stop("'x' must be a list")
  K <- length(x)
  if (K < 2L) stop("'x' must have at least 2 elements (need >= 2 states)")

  if (is.null(names)) {
    names <- as.character(seq_len(K))
  } else {
    if (!is.character(names) || length(names) != K) {
      stop("'names' must be a character vector of length ", K)
    }
    if (anyDuplicated(names)) stop("'names' must be unique")
  }

  M <- matrix(NA_integer_, nrow = K, ncol = K)
  dimnames(M) <- list(from = names, to = names)

  trans_id <- 1L
  for (from in seq_len(K)) {
    targets <- x[[from]]
    if (is.null(targets) || length(targets) == 0L) next

    if (!is.numeric(targets) || any(targets != as.integer(targets))) {
      stop("element ", from, " of 'x' must contain integer state indices")
    }
    targets <- as.integer(targets)

    if (any(targets < 1L | targets > K)) {
      stop("element ", from, " of 'x' contains state index outside 1:", K)
    }
    if (any(targets == from)) {
      stop("self-transition ", from, " -> ", from,
           " is not allowed (states must change)")
    }
    if (anyDuplicated(targets)) {
      stop("element ", from, " of 'x' has duplicated target states")
    }

    for (to in targets) {
      M[from, to] <- trans_id
      trans_id <- trans_id + 1L
    }
  }

  if (trans_id == 1L) stop("no transitions defined")

  M
}

# Helper: build a small Haz data frame manually
# tmat: 3 states, transitions 1=1->2, 2=1->3, 3=2->3
make_haz <- function() {
  data.frame(
    time  = c(0.5, 1.0, 1.5,    # trans 1 jumps
              0.8, 2.0,         # trans 2 jumps
              1.2, 1.8),        # trans 3 jumps
    Haz   = c(0.1, 0.25, 0.5,
              0.05, 0.2,
              0.15, 0.4),
    trans = c(1, 1, 1,
              2, 2,
              3, 3)
  )
}

test_that("prodint_AJ output has correct shape and columns", {
  haz <- make_haz()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  P <- prodint_AJ(haz, tmat, predt = 0, h = 1)

  expect_s3_class(P, "data.frame")
  expect_named(P, c("time", "pstate1", "pstate2", "pstate3"))
  expect_equal(nrow(P), length(unique(haz$time)) + 1L)
})

test_that("prodint_AJ rows sum to 1 at every time point", {
  haz <- make_haz()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  P <- prodint_AJ(haz, tmat, predt = 0, h = 1)

  row_sums <- rowSums(P[, paste0("pstate", 1:3)])
  expect_equal(row_sums, rep(1, nrow(P)), tolerance = 1e-10)
})

test_that("prodint_AJ starts at indicator vector e_h", {
  haz <- make_haz()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  for (h in 1:3) {
    P <- prodint_AJ(haz, tmat, predt = 0, h = h)
    expected <- numeric(3); expected[h] <- 1
    expect_equal(unname(unlist(P[1, paste0("pstate", 1:3)])),
                 expected, tolerance = 1e-12)
  }
})

test_that("prodint_AJ probabilities are in [0, 1]", {
  haz <- make_haz()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  P <- prodint_AJ(haz, tmat, predt = 0, h = 1)

  vals <- as.matrix(P[, paste0("pstate", 1:3)])
  expect_true(all(vals >= -1e-12 & vals <= 1 + 1e-12))
})

test_that("prodint_AJ absorbing state is monotonically non-decreasing", {
  haz <- make_haz()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  P <- prodint_AJ(haz, tmat, predt = 0, h = 1)

  # State 3 is absorbing: P(state 3) should never decrease
  expect_true(all(diff(P$pstate3) >= -1e-12))
})

test_that("prodint_AJ predt argument truncates correctly", {
  haz <- make_haz()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  P0 <- prodint_AJ(haz, tmat, predt = 0,   h = 1)
  P1 <- prodint_AJ(haz, tmat, predt = 1.0, h = 1)

  expect_true(all(P1$time >= 1.0))
  expect_true(nrow(P1) < nrow(P0))
})

test_that("prodint_AJ supports recovery (cyclic transitions)", {
  # Recovery model: 1<->2, 1->3, 2->3
  haz <- data.frame(
    time  = c(0.5, 1.5,    # trans 1: 1->2
              1.0, 2.0,    # trans 2: 1->3
              0.8, 1.8,    # trans 3: 2->1 (recovery)
              1.2, 2.5),   # trans 4: 2->3
    Haz   = c(0.1, 0.3,
              0.05, 0.15,
              0.08, 0.2,
              0.1, 0.25),
    trans = c(1, 1, 2, 2, 3, 3, 4, 4)
  )
  tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)))

  P <- prodint_AJ(haz, tmat, predt = 0, h = 1)

  expect_equal(rowSums(P[, paste0("pstate", 1:3)]),
               rep(1, nrow(P)), tolerance = 1e-10)

  # State 3 (Dead) is still absorbing -> non-decreasing
  expect_true(all(diff(P$pstate3) >= -1e-12))

  # Probability of being in state 1 (Healthy) need not be monotone --
  # this is the whole point of recovery. Just check it dips below 1.
  expect_true(min(P$pstate1) < 1)
})

test_that("prodint_AJ rejects invalid input", {
  haz <- make_haz()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_error(prodint_AJ(list(), tmat), "data.frame")
  expect_error(prodint_AJ(haz, "not a matrix"), "matrix")
  expect_error(prodint_AJ(haz, tmat, h = 99), "h %in%")
  expect_error(prodint_AJ(haz, tmat, predt = c(0, 1)), "length\\(predt\\)")

  bad <- haz; bad$trans[1] <- 99
  expect_error(prodint_AJ(bad, tmat), "not present in tmat")
})

# Helper: build a minimal long-format dataset with known transitions
make_simple_msd <- function() {
  # 5 subjects in illness-death model
  # transitions: 1 = H->I, 2 = H->D, 3 = I->D
  # subj 1: censored healthy at 2.0
  # subj 2: H->D at 1.5
  # subj 3: H->I at 1.0, then I->D at 2.5
  # subj 4: H->I at 0.5, censored ill at 3.0
  # subj 5: censored healthy at 4.0
  data.frame(
    id     = c(1,1, 2,2, 3,3,3, 4,4,4, 5,5),
    Tstart = c(0,0, 0,0, 0,0,1.0, 0,0,0.5, 0,0),
    Tstop  = c(2.0,2.0, 1.5,1.5, 1.0,1.0,2.5, 0.5,0.5,3.0, 4.0,4.0),
    from   = c(1,1, 1,1, 1,1,2, 1,1,2, 1,1),
    to     = c(2,3, 2,3, 2,3,3, 2,3,3, 2,3),
    trans  = c(1,2, 1,2, 1,2,3, 1,2,3, 1,2),
    status = c(0,0, 0,1, 1,0,1, 1,0,0, 0,0)
  )
}

test_that("fit_chaz returns expected columns and types", {
  msd <- make_simple_msd()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  haz <- fit_chaz(msd, tmat)

  expect_s3_class(haz, "data.frame")
  expect_named(haz, c("time", "Haz", "trans"))
  expect_type(haz$trans, "integer")
  expect_true(all(haz$Haz >= 0))
})

test_that("fit_chaz output is sorted by transition then time", {
  msd <- make_simple_msd()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  haz <- fit_chaz(msd, tmat)

  expect_equal(haz$trans, sort(haz$trans))
  for (tr in unique(haz$trans)) {
    times_tr <- haz$time[haz$trans == tr]
    expect_equal(times_tr, sort(times_tr))
  }
})

test_that("fit_chaz cumulative hazards are monotonically non-decreasing", {
  msd <- make_simple_msd()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  haz <- fit_chaz(msd, tmat)

  for (tr in unique(haz$trans)) {
    h_tr <- haz$Haz[haz$trans == tr]
    expect_true(all(diff(h_tr) >= -1e-12))
  }
})

test_that("fit_chaz contains all expected transition IDs", {
  msd <- make_simple_msd()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  haz <- fit_chaz(msd, tmat)

  expect_setequal(unique(haz$trans), c(1L, 2L, 3L))
})

test_that("fit_chaz weights argument is honored", {
  msd <- make_simple_msd()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  unweighted <- fit_chaz(msd, tmat)
  half_w <- fit_chaz(msd, tmat, weights = rep(0.5, nrow(msd)))

  # With uniform weights of 0.5, hazards should equal unweighted hazards
  # (Breslow estimator is invariant to a global rescaling for stratified
  # Cox without covariates).
  expect_equal(half_w$Haz, unweighted$Haz, tolerance = 1e-10)

  # Non-uniform weights should produce different hazards
  varied_w <- runif(nrow(msd), 0.1, 2)
  varied <- fit_chaz(msd, tmat, weights = varied_w)
  expect_false(isTRUE(all.equal(varied$Haz, unweighted$Haz)))
})

test_that("fit_chaz validates inputs", {
  msd <- make_simple_msd()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_error(fit_chaz(data.frame(x = 1), tmat), "missing columns")
  expect_error(fit_chaz(msd, tmat, weights = 1:3),
               "length nrow")
  expect_error(fit_chaz(msd, tmat, weights = rep(-1, nrow(msd))),
               "non-negative")

  bad <- msd
  bad$trans[1] <- 99
  expect_error(fit_chaz(bad, tmat), "not in tmat")
})

make_simple_fit <- function(B = 50, two_sample = FALSE) {
  set.seed(1)
  rows <- list()
  k <- 1L
  for (c in 1:10) {
    for (i in 1:5) {
      t_ill <- rexp(1, 0.5)
      t_dth <- rexp(1, 0.3)
      if (t_ill < t_dth) {
        rows[[length(rows)+1L]] <- data.frame(
          pid = k, site = c, arm = i %% 2,
          t0 = 0, t1 = t_ill, s0 = 1L, s1 = 2L
        )
        rows[[length(rows)+1L]] <- data.frame(
          pid = k, site = c, arm = i %% 2,
          t0 = t_ill, t1 = t_ill + 0.5, s0 = 2L, s1 = 3L
        )
      } else {
        rows[[length(rows)+1L]] <- data.frame(
          pid = k, site = c, arm = i %% 2,
          t0 = 0, t1 = t_dth, s0 = 1L, s1 = 3L
        )
      }
      k <- k + 1L
    }
  }
  d <- do.call(rbind, rows)
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fmla <- if (two_sample) {
    msm(t0, t1, s0, s1) ~ arm
  } else {
    msm(t0, t1, s0, s1) ~ 1
  }

  patp(fmla, data = d, tmat = tmat,
       id = "pid", cluster = "site",
       h = 1, j = 2, s = 0, B = B, seed = 1)
}


test_that("print.patp prints expected sections for one-sample fit", {
  fit <- make_simple_fit()
  out <- capture.output(print(fit))
  expect_true(any(grepl("Population-Averaged", out)))
  expect_true(any(grepl("Subjects:", out)))
  expect_true(any(grepl("Bootstrap", out)))
  expect_true(any(grepl("Curve summary", out)))
})

test_that("print.patp shows test info for two-sample fit", {
  # suppressWarnings: a bootstrap replicate produces a stratum with no
  # events, so survival::basehaz() -> agsurv() emits the harmless
  # "no non-missing arguments to min; returning Inf" warning. The
  # replicate is absorbed normally by downstream alignment; see the
  # "Bootstrap conventions" section of CLAUDE.md.
  fit <- suppressWarnings(make_simple_fit(two_sample = TRUE))
  out <- capture.output(print(fit))
  expect_true(any(grepl("Two-sample test", out)))
  expect_true(any(grepl("Statistic:", out)))
  expect_true(any(grepl("p-value:", out)))
})

test_that("print.patp signals B = 0 explicitly", {
  fit <- make_simple_fit(B = 0)
  out <- capture.output(print(fit))
  expect_true(any(grepl("B = 0", out)))
  expect_true(any(grepl("not computed", out)))
})

test_that("summary.patp returns a summary.patp object", {
  fit <- make_simple_fit()
  s <- summary(fit)
  expect_s3_class(s, "summary.patp")
  expect_true("curves" %in% names(s))
  expect_true("estimand" %in% names(s))
})

test_that("print.summary.patp prints the full curves", {
  fit <- make_simple_fit()
  s <- summary(fit)
  out <- capture.output(print(s))
  expect_true(any(grepl("Summary of patp fit", out)))
  expect_true(any(grepl("Full curve", out)))
})

test_that("print.patp works with no clustering (n_clusters = NA)", {
  set.seed(2)
  d <- data.frame(
    pid = 1:20,
    t0  = 0,
    t1  = rexp(20, 0.5),
    s0  = 1L,
    s1  = 2L
  )
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  fit <- patp(msm(t0, t1, s0, s1) ~ 1,
              data = d, tmat = tmat, id = "pid",
              h = 1, j = 2, s = 0, B = 30, seed = 2)
  out <- capture.output(print(fit))
  expect_true(any(grepl("no clustering", out)))
})

# Helper: build a clustered illness-death dataset in interval format.
#
# `design` controls the cluster/group structure of `treatment`:
#   * "case_i"   (default): every cluster carries both groups
#                (groups mixed within cluster). Bakoyannis (2021).
#   * "case_ii":  each cluster carries exactly one group
#                (cluster-randomized). Bakoyannis & Bandyopadhyay (2022).
#   * "case_iii": some clusters mixed, some single-group (NOT supported
#                by patp(); used to test validation errors).
make_interval_data <- function(n_clusters = 20, per_cluster = 8, seed = 1,
                               design = c("case_i", "case_ii", "case_iii")) {
  design <- match.arg(design)
  set.seed(seed)
  rows <- list()
  k <- 1L
  for (c in seq_len(n_clusters)) {
    cluster_grp <- c %% 2     # used for case_ii and to seed case_iii decisions
    for (i in seq_len(per_cluster)) {
      grp <- switch(design,
        case_i   = i %% 2,
        case_ii  = cluster_grp,
        # case_iii: even-numbered clusters mixed (alternate by subject),
        # odd-numbered clusters single-group (cluster_grp).
        case_iii = if (c %% 2 == 0L) i %% 2 else cluster_grp
      )
      ill_rate <- if (grp == 1) 0.8 else 0.4
      t_ill   <- rexp(1, rate = ill_rate)
      t_dth_h <- rexp(1, rate = 0.3)
      t_dth_i <- rexp(1, rate = 0.6)
      t_cens  <- runif(1, 1, 5)

      ill_first <- t_ill < t_dth_h && t_ill < t_cens
      dth_first <- t_dth_h < t_ill && t_dth_h < t_cens

      if (ill_first) {
        # H -> I, then I -> D or censored ill
        rows[[length(rows) + 1L]] <- data.frame(
          pid = k, site = c, treatment = grp,
          t0 = 0, t1 = t_ill, s0 = 1L, s1 = 2L
        )
        dth_t <- t_ill + t_dth_i
        if (dth_t < t_cens) {
          rows[[length(rows) + 1L]] <- data.frame(
            pid = k, site = c, treatment = grp,
            t0 = t_ill, t1 = dth_t, s0 = 2L, s1 = 3L
          )
        } else {
          rows[[length(rows) + 1L]] <- data.frame(
            pid = k, site = c, treatment = grp,
            t0 = t_ill, t1 = t_cens, s0 = 2L, s1 = 2L
          )
        }
      } else if (dth_first) {
        rows[[length(rows) + 1L]] <- data.frame(
          pid = k, site = c, treatment = grp,
          t0 = 0, t1 = t_dth_h, s0 = 1L, s1 = 3L
        )
      } else {
        # Censored healthy
        rows[[length(rows) + 1L]] <- data.frame(
          pid = k, site = c, treatment = grp,
          t0 = 0, t1 = t_cens, s0 = 1L, s1 = 1L
        )
      }
      k <- k + 1L
    }
  }
  do.call(rbind, rows)
}


test_that("patp one-sample formula returns expected structure", {
  d <- make_interval_data()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ 1,
              data = d, tmat = tmat,
              id = "pid", cluster = "site",
              h = 1, j = 2, s = 0, B = 50, seed = 1)

  expect_s3_class(fit, "patp")
  expect_named(fit, c("curves", "test", "n_clusters", "groups",
                      "call", "formula", "h", "j", "s", "B",
                      "n_subjects"),
               ignore.order = TRUE)
  expect_null(fit$test)
  expect_true(all(c("time", "P", "se", "ll", "ul") %in% names(fit$curves)))
})

test_that("patp two-sample formula returns test slot", {
  d <- make_interval_data()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ treatment,
              data = d, tmat = tmat,
              id = "pid", cluster = "site",
              h = 1, j = 2, s = 0, B = 50, seed = 1)

  expect_s3_class(fit, "patp")
  expect_false(is.null(fit$test))
  expect_named(fit$test, c("statistic", "p.value", "type"))
  expect_true(fit$test$p.value >= 0 && fit$test$p.value <= 1)
  expect_true("group" %in% names(fit$curves))
  expect_setequal(unique(fit$curves$group), c(0, 1))
})

test_that("patp B = 0 returns point estimate only", {
  d <- make_interval_data()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ 1,
              data = d, tmat = tmat, id = "pid",
              h = 1, j = 2, s = 0, B = 0)

  expect_named(fit$curves, c("time", "P"))
  expect_equal(fit$B, 0)
})

test_that("patp errors on two-sample formula with B = 0", {
  d <- make_interval_data()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_error(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat, id = "pid",
         h = 1, j = 2, s = 0, B = 0),
    "two-sample formulas require B > 0"
  )
})

test_that("patp errors on weighted = TRUE without cluster", {
  d <- make_interval_data()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_error(
    patp(msm(t0, t1, s0, s1) ~ 1,
         data = d, tmat = tmat, id = "pid",
         h = 1, j = 2, s = 0, weighted = TRUE, B = 50),
    "weighted = TRUE requires"
  )
})

test_that("patp without cluster uses individual-level bootstrap", {
  d <- make_interval_data(n_clusters = 5, per_cluster = 5)
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ 1,
              data = d, tmat = tmat, id = "pid",
              h = 1, j = 2, s = 0, B = 50, seed = 1)

  expect_true(is.na(fit$n_clusters))
  expect_true(all(c("se", "ll", "ul") %in% names(fit$curves)))
})

test_that("patp probabilities are in [0, 1]", {
  d <- make_interval_data()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ 1,
              data = d, tmat = tmat, id = "pid", cluster = "site",
              h = 1, j = 2, s = 0, B = 0)

  expect_true(all(fit$curves$P >= -1e-10 & fit$curves$P <= 1 + 1e-10))
})

test_that("patp seed produces reproducible results", {
  d <- make_interval_data()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit1 <- patp(msm(t0, t1, s0, s1) ~ 1,
               data = d, tmat = tmat,
               id = "pid", cluster = "site",
               h = 1, j = 2, s = 0, B = 30, seed = 42)
  fit2 <- patp(msm(t0, t1, s0, s1) ~ 1,
               data = d, tmat = tmat,
               id = "pid", cluster = "site",
               h = 1, j = 2, s = 0, B = 30, seed = 42)

  expect_equal(fit1$curves$se, fit2$curves$se)
})

test_that("patp two-sample detects substantial group differences", {
  d <- make_interval_data(n_clusters = 30, per_cluster = 12, seed = 3)
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ treatment,
              data = d, tmat = tmat,
              id = "pid", cluster = "site",
              h = 1, j = 2, s = 0, B = 200, seed = 3)

  # Group 1 has illness hazard 2x group 0
  expect_lt(fit$test$p.value, 0.10)
})

test_that("patp design = 'shared' works on case (i) data", {
  d <- make_interval_data(design = "case_i")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ treatment,
              data = d, tmat = tmat,
              id = "pid", cluster = "site",
              h = 1, j = 2, s = 0, B = 50, seed = 1,
              design = "shared")

  expect_false(is.null(fit$test))
  expect_identical(fit$design, "shared")
})

test_that("patp design = 'cluster_random' works on case (ii) data", {
  d <- make_interval_data(design = "case_ii")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ treatment,
              data = d, tmat = tmat,
              id = "pid", cluster = "site",
              h = 1, j = 2, s = 0, B = 50, seed = 1,
              design = "cluster_random")

  expect_false(is.null(fit$test))
  expect_identical(fit$design, "cluster_random")
})

test_that("patp design = 'indep_random' works on case (ii) data", {
  d <- make_interval_data(design = "case_ii")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ treatment,
              data = d, tmat = tmat,
              id = "pid", cluster = "site",
              h = 1, j = 2, s = 0, B = 50, seed = 1,
              design = "indep_random")

  expect_false(is.null(fit$test))
  expect_identical(fit$design, "indep_random")
})

test_that("patp design = 'auto' picks 'shared' on case (i) data", {
  d <- make_interval_data(design = "case_i")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- patp(msm(t0, t1, s0, s1) ~ treatment,
              data = d, tmat = tmat,
              id = "pid", cluster = "site",
              h = 1, j = 2, s = 0, B = 50, seed = 1)

  expect_identical(fit$design, "shared")
})

test_that("patp design = 'auto' picks 'indep_random' on case (ii) data and warns", {
  d <- make_interval_data(design = "case_ii")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_warning(
    fit <- patp(msm(t0, t1, s0, s1) ~ treatment,
                data = d, tmat = tmat,
                id = "pid", cluster = "site",
                h = 1, j = 2, s = 0, B = 50, seed = 1),
    "set design = 'cluster_random' explicitly"
  )
  expect_identical(fit$design, "indep_random")
})

test_that("patp design = 'auto' on case (i) does not warn", {
  d <- make_interval_data(design = "case_i")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_warning(
    fit <- patp(msm(t0, t1, s0, s1) ~ treatment,
                data = d, tmat = tmat,
                id = "pid", cluster = "site",
                h = 1, j = 2, s = 0, B = 50, seed = 1),
    NA
  )
  expect_identical(fit$design, "shared")
})

test_that("patp design = 'indep_random' explicit does not warn", {
  d <- make_interval_data(design = "case_ii")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_warning(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat,
         id = "pid", cluster = "site",
         h = 1, j = 2, s = 0, B = 50, seed = 1,
         design = "indep_random"),
    NA
  )
})

test_that("patp design = 'shared' errors on case (ii) data", {
  d <- make_interval_data(design = "case_ii")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_error(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat,
         id = "pid", cluster = "site",
         h = 1, j = 2, s = 0, B = 50, seed = 1,
         design = "shared"),
    "design = \"shared\" requires every cluster"
  )
})

test_that("patp design = 'cluster_random' errors on case (i) data", {
  d <- make_interval_data(design = "case_i")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_error(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat,
         id = "pid", cluster = "site",
         h = 1, j = 2, s = 0, B = 50, seed = 1,
         design = "cluster_random"),
    "design = \"cluster_random\" requires each cluster"
  )
})

test_that("patp design = 'indep_random' errors on case (i) data", {
  d <- make_interval_data(design = "case_i")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_error(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat,
         id = "pid", cluster = "site",
         h = 1, j = 2, s = 0, B = 50, seed = 1,
         design = "indep_random"),
    "design = \"indep_random\" requires each cluster"
  )
})

test_that("patp errors on case (iii) data for every design", {
  d <- make_interval_data(design = "case_iii")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_error(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat,
         id = "pid", cluster = "site",
         h = 1, j = 2, s = 0, B = 50, seed = 1),
    "Mixed cluster structure"
  )
  expect_error(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat,
         id = "pid", cluster = "site",
         h = 1, j = 2, s = 0, B = 50, seed = 1,
         design = "shared"),
    "design = \"shared\" requires every cluster"
  )
  expect_error(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat,
         id = "pid", cluster = "site",
         h = 1, j = 2, s = 0, B = 50, seed = 1,
         design = "cluster_random"),
    "design = \"cluster_random\" requires each cluster"
  )
  expect_error(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat,
         id = "pid", cluster = "site",
         h = 1, j = 2, s = 0, B = 50, seed = 1,
         design = "indep_random"),
    "design = \"indep_random\" requires each cluster"
  )
})

test_that("patp design = 'cluster_random' errors without cluster column", {
  d <- make_interval_data(design = "case_i")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  expect_error(
    patp(msm(t0, t1, s0, s1) ~ treatment,
         data = d, tmat = tmat,
         id = "pid",                    # no cluster
         h = 1, j = 2, s = 0, B = 50, seed = 1,
         design = "cluster_random"),
    "design = \"cluster_random\" requires a cluster column"
  )
})

test_that("patp runs with cband = TRUE and produces band columns", {
  d <- make_interval_data()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))

  fit <- suppressWarnings(
    patp(msm(t0, t1, s0, s1) ~ 1,
         data = d, tmat = tmat,
         id = "pid", cluster = "site",
         h = 1, j = 2, s = 0, B = 100, cband = TRUE, seed = 1)
  )

  expect_true(all(c("ll.band", "ul.band") %in% names(fit$curves)))
  # Some band values are NA outside trim; some are not
  expect_true(any(!is.na(fit$curves$ll.band)))
})

test_that("trans_mat builds the standard illness-death matrix correctly", {
  M <- trans_mat(list(c(2, 3), 3, integer(0)),
                 names = c("Healthy", "Ill", "Dead"))

  expect_equal(dim(M), c(3, 3))
  expect_equal(M["Healthy", "Ill"], 1L)
  expect_equal(M["Healthy", "Dead"], 2L)
  expect_equal(M["Ill", "Dead"], 3L)
  expect_true(is.na(M["Ill", "Healthy"]))
  expect_true(is.na(M["Dead", "Healthy"]))
  expect_true(is.na(M["Healthy", "Healthy"]))
})

test_that("trans_mat supports recovery (cyclic transitions)", {
  M <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
                 names = c("H", "I", "D"))

  expect_equal(M["I", "H"], 3L)              # recovery transition
  expect_equal(max(M, na.rm = TRUE), 4L)
  expect_false(is.na(M["I", "H"]))
})

test_that("trans_mat assigns IDs in row-major order", {
  M <- trans_mat(list(c(2, 3), c(1, 3), integer(0)))
  expect_equal(M[1, 2], 1L)                  # row 1 transitions first
  expect_equal(M[1, 3], 2L)
  expect_equal(M[2, 1], 3L)                  # row 2 transitions next
})

test_that("trans_mat handles default and custom names", {
  M1 <- trans_mat(list(2, integer(0)))
  expect_equal(dimnames(M1), list(from = c("1", "2"), to = c("1", "2")))

  M2 <- trans_mat(list(2, integer(0)), names = c("A", "B"))
  expect_equal(dimnames(M2), list(from = c("A", "B"), to = c("A", "B")))
})

test_that("trans_mat rejects invalid input", {
  expect_error(trans_mat("not a list"), "must be a list")
  expect_error(trans_mat(list(2)), "at least 2")
  expect_error(trans_mat(list(c(2, 5), integer(0))), "outside 1:2")
  expect_error(trans_mat(list(1, integer(0))), "self-transition")
  expect_error(trans_mat(list(c(2, 2), integer(0))), "duplicated")
  expect_error(trans_mat(list(2, integer(0)), names = c("A")), "length 2")
  expect_error(trans_mat(list(2, integer(0)), names = c("A", "A")), "unique")
  expect_error(trans_mat(list(integer(0), integer(0))), "no transitions")
})

test_that("trans_mat output works as input to existing patp pipeline", {
  M <- trans_mat(list(c(2, 3), 3, integer(0)),
                 names = c("Healthy", "Ill", "Dead"))
  expect_true(is.matrix(M))
  expect_true(is.integer(M[!is.na(M)]))
})

# This file is part of the standard testthat package layout.
# It is the entry point used by R CMD check to run all tests under
# tests/testthat/.

library(testthat)
library(clusteredMSM)

test_check("clusteredMSM")

#' Construct a Multistate Interval Object for Use in patp() Formulas
#'
#' Packages four columns -- interval start time, interval end time,
#' starting state, and ending state -- into a single object suitable
#' for use as the response in a \code{\link{patp}} formula.
#' Conceptually analogous to \code{\link[survival]{Surv}()} in survival
#' analysis: the user calls it inside a formula
#' (\code{msm(Tstart, Tstop, Sstart, Sstop) ~ ...}).
#'
#' @param Tstart Numeric vector. Start time of each interval.
#' @param Tstop Numeric vector. End time of each interval.
#' @param Sstart Integer-valued numeric vector. State occupied at
#'   \code{Tstart}. States must be whole numbers; the matrix backing
#'   stores them as double (matrices have a single storage mode), but
#'   values are validated to be integer-valued.
#' @param Sstop Integer-valued numeric vector. State occupied at
#'   \code{Tstop}. Same storage caveat as \code{Sstart}. If
#'   \code{Sstart == Sstop}, the row represents a censored interval;
#'   this is permitted only on the final row of a subject's record.
#'
#' @return A four-column matrix of class \code{"msm"} with columns
#'   \code{Tstart}, \code{Tstop}, \code{Sstart}, \code{Sstop}.
#'
#' @details
#' Used inside a model formula passed to \code{\link{patp}}, e.g.
#' \code{patp(msm(Tstart, Tstop, Sstart, Sstop) ~ 1, ...)}. The four
#' arguments can be named anything in the user's data -- the formula
#' machinery looks them up in the supplied \code{data}.
#'
#' @examples
#' Tstart <- c(0,   1.5, 0,   0)
#' Tstop  <- c(1.5, 3.0, 2.0, 1.2)
#' Sstart <- c(1,   2,   1,   1)
#' Sstop  <- c(2,   3,   3,   1)   # last row censored (S unchanged)
#' obj <- msm(Tstart, Tstop, Sstart, Sstop)
#' head(obj)
#'
#' @seealso \code{\link{patp}}, \code{\link{validate_intervals}}.
#' @export
msm <- function(Tstart, Tstop, Sstart, Sstop) {

  if (missing(Tstart) || missing(Tstop) ||
      missing(Sstart) || missing(Sstop)) {
    stop("msm() requires four arguments: Tstart, Tstop, Sstart, Sstop")
  }

  n <- length(Tstart)
  if (length(Tstop)  != n || length(Sstart) != n || length(Sstop) != n) {
    stop("All arguments to msm() must have the same length")
  }
  if (!is.numeric(Tstart) || !is.numeric(Tstop)) {
    stop("Tstart and Tstop must be numeric")
  }
  if (any(Tstart >= Tstop, na.rm = TRUE)) {
    stop("Tstart must be strictly less than Tstop on every row")
  }
  if (!is.numeric(Sstart) || !is.numeric(Sstop)) {
    stop("Sstart and Sstop must be numeric (integer-valued)")
  }
  if (any(Sstart != as.integer(Sstart), na.rm = TRUE) ||
      any(Sstop  != as.integer(Sstop),  na.rm = TRUE)) {
    stop("Sstart and Sstop must be integer-valued (whole numbers)")
  }

  out <- cbind(Tstart = Tstart, Tstop = Tstop,
               Sstart = Sstart, Sstop = Sstop)
  class(out) <- c("msm", class(out))
  out
}


#' @export
print.msm <- function(x, ...) {
  cat("msm object: ", nrow(x), " interval",
      if (nrow(x) != 1L) "s" else "", "\n", sep = "")
  print(unclass(utils::head(x, 6L)), ...)
  if (nrow(x) > 6L) cat("... (", nrow(x) - 6L, " more)\n", sep = "")
  invisible(x)
}

make_test_data <- function() {
  data.frame(
    pid    = c(1, 1, 2, 3),
    site   = c(1, 1, 1, 2),
    arm    = c("A", "A", "B", "B"),
    t0     = c(0,   0.5, 0,   0),
    t1     = c(0.5, 1.5, 1.0, 2.0),
    s0     = c(1,   2,   1,   1),
    s1     = c(2,   3,   3,   1),
    stringsAsFactors = FALSE
  )
}

test_that("parse_msm_formula handles one-sample formulas", {
  d <- make_test_data()
  out <- parse_msm_formula(msm(t0, t1, s0, s1) ~ 1, d, id = "pid")

  expect_named(out, c("data", "group", "has_cluster", "group_name"))
  expect_null(out$group)
  expect_false(out$has_cluster)
  expect_named(out$data, c("id", "Tstart", "Tstop", "Sstart", "Sstop"))
  expect_equal(out$data$Tstart, c(0, 0.5, 0, 0))
})

test_that("parse_msm_formula handles two-sample formulas", {
  d <- make_test_data()
  out <- parse_msm_formula(msm(t0, t1, s0, s1) ~ arm, d, id = "pid")

  expect_equal(out$group, c("A", "A", "B", "B"))
  expect_equal(out$group_name, "arm")
  expect_true("group" %in% names(out$data))
})

test_that("parse_msm_formula picks up cluster column when supplied", {
  d <- make_test_data()
  out <- parse_msm_formula(msm(t0, t1, s0, s1) ~ 1, d,
                           id = "pid", cluster = "site")

  expect_true(out$has_cluster)
  expect_true("cluster" %in% names(out$data))
  expect_equal(out$data$cluster, c(1, 1, 1, 2))
})

test_that("parse_msm_formula treats NA cluster as absent", {
  d <- make_test_data()
  out <- parse_msm_formula(msm(t0, t1, s0, s1) ~ 1, d,
                           id = "pid", cluster = NA)
  expect_false(out$has_cluster)
  expect_false("cluster" %in% names(out$data))
})

test_that("parse_msm_formula errors when LHS is not msm()", {
  d <- make_test_data()
  expect_error(parse_msm_formula(t0 ~ 1, d, id = "pid"),
               "left-hand side must be a call to msm")
  expect_error(parse_msm_formula(Surv(t0, t1) ~ 1, d, id = "pid"),
               "left-hand side must be a call to msm")
})

test_that("parse_msm_formula errors on wrong number of msm arguments", {
  d <- make_test_data()
  expect_error(parse_msm_formula(msm(t0, t1, s0) ~ 1, d, id = "pid"),
               "exactly 4 arguments")
})

test_that("parse_msm_formula errors when id missing or invalid", {
  d <- make_test_data()
  expect_error(parse_msm_formula(msm(t0, t1, s0, s1) ~ 1, d, id = NULL),
               "single character string")
  expect_error(parse_msm_formula(msm(t0, t1, s0, s1) ~ 1, d,
                                 id = "missing_col"),
               "not found")
})

test_that("parse_msm_formula errors on multiple RHS terms", {
  d <- make_test_data()
  d$z <- 1:4
  expect_error(parse_msm_formula(msm(t0, t1, s0, s1) ~ arm + z, d,
                                 id = "pid"),
               "single grouping variable")
})

test_that("parse_msm_formula errors when group has != 2 levels", {
  d <- make_test_data()
  d$arm <- "A"
  expect_error(parse_msm_formula(msm(t0, t1, s0, s1) ~ arm, d, id = "pid"),
               "exactly 2 distinct")
})

test_that("parse_msm_formula errors when grouping variable not in data", {
  d <- make_test_data()
  expect_error(parse_msm_formula(msm(t0, t1, s0, s1) ~ ghost, d, id = "pid"),
               "not found")
})

test_that("msm constructs a valid object with correct columns", {
  obj <- msm(c(0, 1.5), c(1.5, 3.0), c(1, 2), c(2, 3))

  expect_s3_class(obj, "msm")
  expect_true(is.matrix(obj))
  expect_equal(colnames(obj), c("Tstart", "Tstop", "Sstart", "Sstop"))
  expect_equal(nrow(obj), 2)
})

test_that("msm accepts integer-valued states and rejects fractional ones", {
  # Whole-number doubles are accepted (matrix storage is double; values
  # are validated to be integer-valued).
  expect_silent(msm(c(0, 1), c(1, 2), c(1.0, 2.0), c(2.0, 3.0)))
  # Fractional state values are rejected.
  expect_error(msm(c(0, 1), c(1, 2), c(1.5, 2.0), c(2.0, 3.0)),
               "integer-valued")
})

test_that("msm errors on length mismatch", {
  expect_error(msm(c(0, 1), c(1, 2, 3), c(1, 1), c(2, 2)),
               "same length")
})

test_that("msm errors on missing arguments", {
  expect_error(msm(), "four arguments")
  expect_error(msm(0), "four arguments")
})

test_that("msm errors when Tstart >= Tstop", {
  expect_error(msm(c(0, 2), c(1, 1), c(1, 1), c(2, 2)),
               "strictly less than")
  expect_error(msm(c(0), c(0), c(1), c(2)),
               "strictly less than")
})

test_that("msm errors on non-numeric times", {
  expect_error(msm(c("a", "b"), c(1, 2), c(1, 1), c(2, 2)),
               "numeric")
})

test_that("msm.print runs without error", {
  obj <- msm(c(0, 1, 2), c(1, 2, 3), c(1, 1, 2), c(1, 2, 3))
  expect_output(print(obj), "msm object")
})

test_that("msm allows censoring rows (Sstart == Sstop)", {
  # Censoring is permitted at the constructor level; it's the
  # validator's job to enforce that it's only on the last row.
  obj <- msm(0, 1, 1, 1)
  expect_equal(unname(obj[1, "Sstart"]), 1L)
  expect_equal(unname(obj[1, "Sstop"]), 1L)
})

## Internal orchestration functions for patp().
##
## Not exported. These are split out so the public patp() function
## stays a thin formula-parsing dispatcher.

## ---- One-sample ----------------------------------------------------------

.patp_onesample <- function(long, tmat, has_cluster,
                            h, j, s,
                            weighted, LMAJ,
                            B, cband, level, seed) {

  resample_col <- if (has_cluster) "cluster" else "id"

  if (weighted) long <- add_cluster_sizes(long, "cluster", "id")

  ## Point estimate
  point <- .patp_point(long, tmat, h = h, j = j, s = s,
                       weighted = weighted, lmaj = LMAJ)

  curves <- data.frame(time = point$time, P = point$pstate)

  if (B > 0L) {
    grid <- point$time
    init <- if (h == j) 1 else 0

    fn_boot <- function(boot_data, ...) {
      pb <- tryCatch(
        .patp_point(boot_data, tmat, h = h, j = j, s = s,
                    weighted = weighted, lmaj = LMAJ),
        error = function(e) NULL
      )
      if (is.null(pb)) return(rep(NA_real_, length(grid)))
      step_interp(pb$time, pb$pstate, grid, init = init)
    }

    boot_mat <- cluster_boot(long, cluster = resample_col, B = B,
                             fn = fn_boot, seed = seed)

    se <- apply(boot_mat, 1L, stats::sd, na.rm = TRUE)
    ci <- ci_cloglog(curves$P, se, level = level)

    curves$se <- se
    curves$ll <- ci$ll
    curves$ul <- ci$ul

    if (cband) {
      band <- confidence_band(curves$P, boot_mat,
                              times = grid, level = level)
      curves$ll.band <- band$ll.band
      curves$ul.band <- band$ul.band
    }
  }

  list(curves     = curves,
       test       = NULL,
       n_clusters = if (has_cluster) length(unique(long$cluster)) else NA_integer_,
       groups     = NULL)
}


## ---- Two-sample ---------------------------------------------------------

.patp_twosample <- function(long, tmat, has_cluster,
                            h, j, s,
                            weighted, LMAJ,
                            B, cband, level, seed,
                            group_name,
                            design = c("auto", "shared",
                                       "cluster_random", "indep_random")) {

  design <- match.arg(design)
  resample_col <- if (has_cluster) "cluster" else "id"

  if (weighted) long <- add_cluster_sizes(long, "cluster", "id")

  group_levels <- sort(unique(long$group))
  long_0 <- long[long$group == group_levels[1L], , drop = FALSE]
  long_1 <- long[long$group == group_levels[2L], , drop = FALSE]

  ## ---- Resolve / validate design against the cluster/group structure ----
  ##
  ## Three regimes:
  ##   shared         every cluster carries both groups (case i,
  ##                  Bakoyannis 2021).
  ##   cluster_random each cluster carries one group, n_1 and n_2 fixed
  ##                  by design (case ii.a, Bakoyannis & Bandyopadhyay
  ##                  2022). Stratified bootstrap.
  ##   indep_random   each cluster carries one group, n_1 and n_2 random
  ##                  (independent observational comparison; case ii.b).
  ##                  Unstratified bootstrap.
  ##
  ## The two-sample asymptotic regime applies for both ii.a and ii.b, so
  ## the scaling factor is sqrt(n_1 n_2 / (n_1 + n_2)) for either; only
  ## the bootstrap (stratified vs. unstratified) differs.
  ##
  ## "auto" never picks "cluster_random" -- that is a stronger claim
  ## about the data-generating process and must be opted into by the
  ## user.
  if (has_cluster) {
    cluster_groups <- tapply(long$group, long$cluster,
                             function(x) length(unique(x)))
    n_one  <- sum(cluster_groups == 1L)
    n_both <- sum(cluster_groups == 2L)
  } else {
    n_one  <- NA_integer_
    n_both <- NA_integer_
  }

  if (design == "auto") {
    if (!has_cluster) {
      design <- "indep_random"
    } else if (n_one == 0L) {
      design <- "shared"
    } else if (n_both == 0L) {
      design <- "indep_random"            # safer default than cluster_random
      warning("Each cluster contains observations from only one group. ",
              "Using design = 'indep_random' (independent observational ",
              "comparison, unstratified bootstrap). If this is a ",
              "cluster-randomized trial with fixed n_1 and n_2, set ",
              "design = 'cluster_random' explicitly for stratified ",
              "resampling.",
              call. = FALSE)
    } else {
      stop("Mixed cluster structure detected (some clusters carry both ",
           "groups, some only one). This regime is not supported in ",
           "the current version.")
    }
  } else if (design == "shared") {
    if (has_cluster && n_one > 0L) {
      stop("design = \"shared\" requires every cluster to contain ",
           "observations from both groups, but some clusters contain ",
           "only one. If this is a cluster-randomized trial, use ",
           "design = \"cluster_random\"; if it is an independent ",
           "observational comparison, use design = \"indep_random\".")
    }
  } else if (design == "cluster_random") {
    if (!has_cluster) {
      stop("design = \"cluster_random\" requires a cluster column. ",
           "Cluster-randomized inference is defined at the cluster ",
           "level; subject-level stratification would be a different ",
           "procedure.")
    }
    if (n_both > 0L) {
      stop("design = \"cluster_random\" requires each cluster to ",
           "contain observations from only one group, but some ",
           "clusters contain both. Use design = \"shared\" if this ",
           "is a multicenter trial.")
    }
  } else if (design == "indep_random") {
    if (has_cluster && n_both > 0L) {
      stop("design = \"indep_random\" requires each cluster to ",
           "contain observations from only one group, but some ",
           "clusters contain both. Use design = \"shared\" if this ",
           "is a multicenter trial.")
    }
  }

  p0 <- .patp_point(long_0, tmat, h = h, j = j, s = s,
                    weighted = weighted, lmaj = LMAJ)
  p1 <- .patp_point(long_1, tmat, h = h, j = j, s = s,
                    weighted = weighted, lmaj = LMAJ)

  grid <- sort(unique(c(p0$time, p1$time)))
  init <- if (h == j) 1 else 0
  p0_grid <- step_interp(p0$time, p0$pstate, grid, init = init)
  p1_grid <- step_interp(p1$time, p1$pstate, grid, init = init)
  diff_point <- p1_grid - p0_grid

  fn_diff <- function(boot_data, ...) {
    d0 <- boot_data[boot_data$group == group_levels[1L], , drop = FALSE]
    d1 <- boot_data[boot_data$group == group_levels[2L], , drop = FALSE]

    pb0 <- tryCatch(
      .patp_point(d0, tmat, h = h, j = j, s = s,
                  weighted = weighted, lmaj = LMAJ),
      error = function(e) NULL
    )
    pb1 <- tryCatch(
      .patp_point(d1, tmat, h = h, j = j, s = s,
                  weighted = weighted, lmaj = LMAJ),
      error = function(e) NULL
    )
    if (is.null(pb0) || is.null(pb1)) {
      return(rep(NA_real_, length(grid)))
    }

    g1 <- step_interp(pb1$time, pb1$pstate, grid, init = init)
    g0 <- step_interp(pb0$time, pb0$pstate, grid, init = init)
    g1 - g0
  }

  fn_curves <- function(boot_data, ...) {
    d0 <- boot_data[boot_data$group == group_levels[1L], , drop = FALSE]
    d1 <- boot_data[boot_data$group == group_levels[2L], , drop = FALSE]
    pb0 <- tryCatch(
      .patp_point(d0, tmat, h = h, j = j, s = s,
                  weighted = weighted, lmaj = LMAJ),
      error = function(e) NULL
    )
    pb1 <- tryCatch(
      .patp_point(d1, tmat, h = h, j = j, s = s,
                  weighted = weighted, lmaj = LMAJ),
      error = function(e) NULL
    )
    g0 <- if (is.null(pb0)) rep(NA_real_, length(grid)) else
            step_interp(pb0$time, pb0$pstate, grid, init = init)
    g1 <- if (is.null(pb1)) rep(NA_real_, length(grid)) else
            step_interp(pb1$time, pb1$pstate, grid, init = init)
    c(g0, g1)
  }

  ## ---- Bootstrap: stratified iff design == "cluster_random" ----
  if (design == "cluster_random") {
    cluster_to_group <- tapply(long$group, long$cluster,
                               function(x) as.character(x[1L]))
    strata_vec <- as.character(cluster_to_group)
    names(strata_vec) <- names(cluster_to_group)
  } else {
    strata_vec <- NULL
  }

  diff_boot   <- cluster_boot(long, cluster = resample_col, B = B,
                              fn = fn_diff, strata = strata_vec,
                              seed = seed)
  curves_boot <- cluster_boot(long, cluster = resample_col, B = B,
                              fn = fn_curves, strata = strata_vec,
                              seed = if (is.null(seed)) NULL else seed + 1L)

  n <- length(unique(long[[resample_col]]))

  ## ---- Asymptotic scaling factor ----
  ## sqrt(n) for case (i) [shared]; sqrt(n_1 n_2 / (n_1 + n_2)) for
  ## both two-independent-sample regimes (ii.a and ii.b). The factor
  ## is determined by the asymptotic regime, not by whether the
  ## bootstrap is stratified.
  if (design == "shared") {
    scale_factor <- sqrt(n)
  } else {
    n1 <- length(unique(long_0[[resample_col]]))
    n2 <- length(unique(long_1[[resample_col]]))
    scale_factor <- sqrt(n1 * n2 / (n1 + n2))
  }

  ## Per-group SEs and CIs
  ng <- length(grid)
  boot_0 <- curves_boot[seq_len(ng), , drop = FALSE]
  boot_1 <- curves_boot[(ng + 1L):(2L * ng), , drop = FALSE]
  se_0   <- apply(boot_0, 1L, stats::sd, na.rm = TRUE)
  se_1   <- apply(boot_1, 1L, stats::sd, na.rm = TRUE)
  ci_0   <- ci_cloglog(p0_grid, se_0, level = level)
  ci_1   <- ci_cloglog(p1_grid, se_1, level = level)

  curves <- rbind(
    data.frame(time = grid, P = p0_grid,
               se = se_0, ll = ci_0$ll, ul = ci_0$ul,
               group = group_levels[1L], stringsAsFactors = FALSE),
    data.frame(time = grid, P = p1_grid,
               se = se_1, ll = ci_1$ll, ul = ci_1$ul,
               group = group_levels[2L], stringsAsFactors = FALSE)
  )

  if (cband) {
    band_0 <- confidence_band(p0_grid, boot_0, times = grid, level = level)
    band_1 <- confidence_band(p1_grid, boot_1, times = grid, level = level)
    curves$ll.band <- c(band_0$ll.band, band_1$ll.band)
    curves$ul.band <- c(band_0$ul.band, band_1$ul.band)
  }

  test <- ks_pvalue(diff_point, diff_boot, scale = scale_factor)

  list(curves       = curves,
       test         = list(statistic = test$statistic,
                           p.value   = test$p.value,
                           type      = "Kolmogorov-Smirnov"),
       n_clusters   = if (has_cluster) length(unique(long$cluster)) else NA_integer_,
       groups       = group_levels,
       group_name   = group_name,
       design       = design)
}


## ---- The bare point estimator (formerly exported as patp_point) ----

.patp_point <- function(data, tmat, h, j, s = 0,
                        weighted = FALSE, lmaj = FALSE) {
  if (lmaj) return(.patp_lmaj(data, tmat, h, j, s, weighted))

  w <- if (weighted) {
    if (!"clust.size" %in% names(data)) {
      stop("weighted = TRUE requires a 'clust.size' column")
    }
    1 / data$clust.size
  } else NULL

  haz <- fit_chaz(data, tmat, weights = w)
  P   <- prodint_AJ(haz, tmat, predt = s, h = h)
  out <- P[, c("time", paste0("pstate", j))]
  names(out) <- c("time", "pstate")
  out
}


.patp_lmaj <- function(data, tmat, h, j, s, weighted = FALSE) {
  if (!"id" %in% names(data)) {
    stop("'data' must contain an 'id' column for landmark estimation")
  }
  in_h <- state_at(data, s, id = "id")
  in_h_ids <- in_h[["id"]][in_h$state %in% h]
  if (length(in_h_ids) == 0L) {
    stop("no subjects in state(s) ", paste(h, collapse = ","),
         " at time s = ", s)
  }
  data_lm <- cut_at_lm(data, s)
  data_lm <- data_lm[data_lm[["id"]] %in% in_h_ids, , drop = FALSE]

  w <- if (weighted) {
    if (!"clust.size" %in% names(data_lm)) {
      stop("weighted = TRUE requires a 'clust.size' column")
    }
    1 / data_lm$clust.size
  } else NULL

  haz <- fit_chaz(data_lm, tmat, weights = w)
  P   <- prodint_AJ(haz, tmat, predt = s, h = h)
  out <- P[, c("time", paste0("pstate", j))]
  names(out) <- c("time", "pstate")
  out
}

#' Validate Independent-Clusters Assumption
#'
#' Checks that no individual ID appears in more than one cluster.
#' Stops with an informative error if the assumption is violated.
#'
#' @param data A data frame.
#' @param cid Character. Cluster ID column name.
#' @param id Character. Individual ID column name.
#'
#' @keywords internal
check_clusters <- function(data, cid, id) {
  by_id <- stats::aggregate(data[[cid]], by = list(data[[id]]),
                            FUN = function(x) length(unique(x)))
  bad <- by_id$x > 1
  if (any(bad)) {
    stop("same individual(s) appear in more than 1 cluster ",
         "(violation of the independent clusters assumption)")
  }
  invisible(TRUE)
}


#' Add Cluster-Size Column for Inverse-Cluster-Size Weighting
#'
#' Computes the number of distinct individuals per cluster and merges
#' it back into the long-format data as a \code{clust.size} column.
#'
#' @param data A long-format data frame containing \code{cid} and \code{id}.
#' @param cid Character. Cluster ID column name.
#' @param id Character. Individual ID column name.
#'
#' @return The input data with an added \code{clust.size} column.
#'
#' @keywords internal
add_cluster_sizes <- function(data, cid, id) {
  per_subject <- stats::aggregate(rep(1, nrow(data)),
                                  by = list(data[[cid]], data[[id]]),
                                  FUN = mean)
  per_cluster <- stats::aggregate(per_subject$x,
                                  by = list(per_subject$Group.1),
                                  FUN = sum)
  names(per_cluster) <- c(cid, "clust.size")

  data <- merge(data, per_cluster, by = cid)
  data <- data[order(data[[cid]], data[[id]]), ]
  rownames(data) <- NULL
  data
}


#' Interpolate a Step Function onto a Time Grid
#'
#' Right-continuous step interpolation of a (time, value) sequence onto
#' a target grid. Used to align bootstrap replicates onto a common time
#' grid so they can be assembled into a matrix.
#'
#' @param times Numeric. Jump times where the function changes.
#' @param values Numeric. Function values; \code{values[k]} is the
#'   value on \code{[times[k], times[k+1])}.
#' @param grid Numeric. Output grid.
#' @param init Numeric scalar. Value to return for grid points before
#'   \code{times[1]}. Default 0.
#'
#' @return Numeric vector of values on \code{grid}.
#'
#' @keywords internal
step_interp <- function(times, values, grid, init = 0) {
  if (length(times) == 0L) return(rep(init, length(grid)))
  fn <- stats::stepfun(times, c(init, values))
  fn(grid)
}

#' Aalen-Johansen Product Integral for General Multistate Processes
#'
#' Computes the row of the transition probability matrix
#' \deqn{P(s, t) = \prod_{u \in (s, t]} (I + dA(u))}
#' starting from state \code{h}, given Nelson-Aalen
#' transition hazard estimates.
#'
#' @param haz_df Data frame with columns \code{time}, \code{Haz},
#'   \code{trans}, typically the output of \code{fit_chaz()} or the
#'   \code{$Haz} element of an \code{mstate::msfit} object. \code{Haz}
#'   is the cumulative (not incremental) hazard.
#' @param tmat A K x K transition matrix from \code{trans_mat()}.
#'   Cyclic transitions (both \code{[h, j]} and \code{[j, h]} non-NA)
#'   are permitted.
#' @param predt Numeric scalar. Starting time s. Probabilities are
#'   computed for t > predt. Default 0.
#' @param h Integer in 1..K. Starting state.
#'
#' @return A data frame with columns \code{time}, \code{pstate1}, ...,
#'   \code{pstateK}. Row k gives \eqn{P(predt, time_k)[h, ]}, the
#'   probabilities of being in each state at \code{time_k} given the
#'   process was in state \code{h} at \code{predt}.
#'
#' @details
#' At each unique jump time \eqn{u}, the increment matrix
#' \eqn{dA(u)} has off-diagonal entry \eqn{(h, j)} equal to the
#' Nelson-Aalen increment of the cumulative hazard for transition
#' \eqn{h \to j}, and diagonal entries equal to the negative row sums.
#' The transition probability matrix is then updated by
#' \eqn{P \leftarrow P (I + dA(u))}.
#'
#' Computational complexity is O(J K^2) where J is the number of unique
#' jump times and K is the number of states.
#'
#' @examples
#' data(example_msm)
#' tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
#'                   names = c("Healthy", "Ill", "Dead"))
#'
#' long <- intervals_to_long(example_msm, tmat)
#' haz  <- fit_chaz(long, tmat)
#' P    <- prodint_AJ(haz, tmat, predt = 0, h = 1)
#' head(P)
#'
#' @export
prodint_AJ <- function(haz_df, tmat, predt = 0, h = 1) {

  ## ---- input checks ----
  stopifnot(is.data.frame(haz_df),
            all(c("time", "Haz", "trans") %in% names(haz_df)))
  stopifnot(is.matrix(tmat), nrow(tmat) == ncol(tmat))
  K <- nrow(tmat)
  stopifnot(length(h) == 1L, h %in% seq_len(K))
  stopifnot(length(predt) == 1L, is.numeric(predt))

  ntrans <- suppressWarnings(max(tmat, na.rm = TRUE))
  if (!is.finite(ntrans)) stop("tmat has no defined transitions")
  if (!all(sort(unique(haz_df$trans)) %in% seq_len(ntrans))) {
    stop("haz_df$trans contains ids not present in tmat")
  }

  ## map each transition id to its (from, to) pair
  idx <- which(!is.na(tmat), arr.ind = TRUE)
  ord <- order(tmat[!is.na(tmat)])
  idx <- idx[ord, , drop = FALSE]
  rownames(idx) <- NULL

  ## ---- restrict to t > predt and find unique jump times ----
  haz_df <- haz_df[haz_df$time > predt, , drop = FALSE]
  haz_df <- haz_df[order(haz_df$trans, haz_df$time), , drop = FALSE]

  jt <- sort(unique(haz_df$time))
  out <- matrix(0, length(jt) + 1L, K)
  P   <- diag(K)
  out[1L, ] <- P[h, ]

  ## previous cumulative hazard per transition (for differencing)
  prev_haz <- numeric(ntrans)

  ## ---- main loop: product integrate over jump times ----
  for (k in seq_along(jt)) {
    rows <- haz_df[haz_df$time == jt[k], , drop = FALSE]
    dA <- matrix(0, K, K)

    for (i in seq_len(nrow(rows))) {
      tr  <- rows$trans[i]
      inc <- rows$Haz[i] - prev_haz[tr]
      if (inc < 0) {
        if (inc < -1e-10) {
          warning("negative hazard increment at t=", jt[k],
                  " trans=", tr)
        }
        inc <- 0
      }
      prev_haz[tr] <- rows$Haz[i]
      dA[idx[tr, 1L], idx[tr, 2L]] <- inc
    }
    diag(dA) <- -rowSums(dA)

    P <- P %*% (diag(K) + dA)
    out[k + 1L, ] <- P[h, ]
  }

  ## ---- assemble output ----
  res <- data.frame(time = c(predt, jt), out)
  names(res)[-1L] <- paste0("pstate", seq_len(K))
  res
}

#' Print Method for patp Objects
#'
#' Concise summary of a \code{\link{patp}} fit. For two-sample fits,
#' includes the K-S statistic and p-value.
#'
#' @param x A \code{patp} object.
#' @param digits Integer. Number of significant digits for printing.
#' @param ... Additional arguments (ignored).
#'
#' @return The object \code{x}, invisibly.
#'
#' @export
print.patp <- function(x, digits = max(3L, getOption("digits") - 3L), ...) {

  cat("\nclusteredMSM: Population-Averaged Transition Probability\n")
  cat(strrep("-", 56), "\n", sep = "")

  cat("Call:\n  ")
  print(x$call)

  cat("\nEstimand: P(X(t) =", x$j, "| X(", x$s, ") =", x$h, ")\n")
  cat("Subjects:", x$n_subjects)
  if (!is.na(x$n_clusters)) {
    cat("   Clusters:", x$n_clusters, "\n")
  } else {
    cat("   (no clustering)\n")
  }

  if (x$B > 0L) {
    cat("Bootstrap replications: B =", x$B, "\n")
  } else {
    cat("Bootstrap: NONE (B = 0; standard errors and CIs not computed)\n")
  }

  if (!is.null(x$test)) {
    cat("\nTwo-sample test (", x$test$type, "):\n", sep = "")
    d_label <- if (is.null(x$design)) "shared" else x$design
    design_str <- switch(d_label,
      shared         = "shared (multicenter, dependent groups)",
      cluster_random = "cluster-randomized (independent groups, fixed n_1, n_2)",
      indep_random   = "independent samples (random n_1, n_2)",
      d_label
    )
    cat("  Design:    ", design_str, "\n", sep = "")
    cat("  Groups:    ", paste(x$groups, collapse = " vs "), "\n", sep = "")
    cat("  Statistic: ", format(x$test$statistic, digits = digits), "\n",
        sep = "")
    cat("  p-value:   ", format.pval(x$test$p.value, digits = digits), "\n",
        sep = "")
  }

  cat("\nCurve summary (", nrow(x$curves), " rows total):\n", sep = "")
  print(utils::head(x$curves, 6L), digits = digits, row.names = FALSE)
  if (nrow(x$curves) > 6L) {
    cat("... (", nrow(x$curves) - 6L, " more rows; access via $curves)\n",
        sep = "")
  }

  invisible(x)
}


#' Summary Method for patp Objects
#'
#' Returns the full curve(s) and (if applicable) the test result.
#'
#' @param object A \code{patp} object.
#' @param ... Additional arguments (ignored).
#'
#' @return A list with named components for further inspection.
#'
#' @export
summary.patp <- function(object, ...) {
  out <- list(
    call         = object$call,
    formula      = object$formula,
    estimand     = sprintf("P(X(t) = %d | X(%g) = %d)",
                           object$j, object$s, object$h),
    n_subjects   = object$n_subjects,
    n_clusters   = object$n_clusters,
    B            = object$B,
    curves       = object$curves,
    test         = object$test,
    groups       = object$groups
  )
  class(out) <- "summary.patp"
  out
}


#' @export
print.summary.patp <- function(x, ...) {
  cat("\nSummary of patp fit\n")
  cat(strrep("-", 56), "\n", sep = "")
  cat("Call:\n  "); print(x$call)
  cat("\nEstimand:  ", x$estimand, "\n", sep = "")
  cat("Subjects:  ", x$n_subjects, "\n", sep = "")
  if (!is.na(x$n_clusters)) cat("Clusters:  ", x$n_clusters, "\n", sep = "")
  cat("Bootstrap: B = ", x$B, "\n", sep = "")
  if (!is.null(x$test)) {
    cat("\nTwo-sample test:\n")
    cat("  Type:      ", x$test$type, "\n")
    cat("  Statistic: ", format(x$test$statistic), "\n")
    cat("  p-value:   ", format.pval(x$test$p.value), "\n")
    cat("  Groups:    ", paste(x$groups, collapse = " vs "), "\n")
  }
  cat("\nFull curve(s):\n")
  print(x$curves, row.names = FALSE)
  invisible(x)
}

#' Strictly Validate Multistate Data in Interval Format
#'
#' Performs strict validation on a data frame of multistate data in interval
#' format (the output of \code{\link{parse_msm_formula}()}). Stops with
#' an informative error on the first violation.
#'
#' @param data A data frame with columns \code{id}, \code{Tstart},
#'   \code{Tstop}, \code{Sstart}, \code{Sstop}.
#' @param tmat A K x K transition matrix from \code{\link{trans_mat}()}.
#' @param tol Numeric tolerance for temporal-contiguity checks.
#'   Default 1e-9.
#'
#' @return Invisibly returns \code{TRUE} if all checks pass. Stops with
#'   an error otherwise.
#'
#' @details
#' Validation rules, applied in order:
#' \enumerate{
#'   \item Required columns are present and of correct type.
#'   \item Every row has \code{Tstart < Tstop}.
#'   \item All states are integers in \code{1..K}.
#'   \item Within each subject, rows are sorted by time.
#'   \item Within each subject, rows are temporally contiguous:
#'     \code{Tstop[k] == Tstart[k+1]}.
#'   \item Within each subject, rows are spatially contiguous:
#'     \code{Sstop[k] == Sstart[k+1]}.
#'   \item No row has \code{Sstart} equal to an absorbing state.
#'   \item No row follows one whose \code{Sstop} is absorbing.
#'   \item Each non-censored transition (\code{Sstart != Sstop}) is
#'     allowed by \code{tmat}.
#'   \item \code{Sstart == Sstop} (censoring) is permitted only on the
#'     final row of a subject's record.
#' }
#'
#' @examples
#' data(example_msm)
#' tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
#'                   names = c("Healthy", "Ill", "Dead"))
#' validate_intervals(example_msm, tmat)
#'
#' @keywords internal
#' @export
validate_intervals <- function(data, tmat, tol = 1e-9) {

  required <- c("id", "Tstart", "Tstop", "Sstart", "Sstop")
  missing_cols <- setdiff(required, names(data))
  if (length(missing_cols)) {
    stop("'data' is missing required columns: ",
         paste(missing_cols, collapse = ", "))
  }

  if (!is.numeric(data$Tstart) || !is.numeric(data$Tstop)) {
    stop("'Tstart' and 'Tstop' must be numeric")
  }
  if (!is.numeric(data$Sstart) || !is.numeric(data$Sstop)) {
    stop("'Sstart' and 'Sstop' must be integer-valued numerics")
  }
  if (any(data$Sstart != as.integer(data$Sstart)) ||
      any(data$Sstop  != as.integer(data$Sstop))) {
    stop("'Sstart' and 'Sstop' must be integer-valued")
  }

  K <- nrow(tmat)
  if (any(data$Sstart < 1L | data$Sstart > K) ||
      any(data$Sstop  < 1L | data$Sstop  > K)) {
    stop("State values must be integers in 1..", K)
  }

  if (any(data$Tstart >= data$Tstop)) {
    bad <- which(data$Tstart >= data$Tstop)[1L]
    stop("Tstart must be < Tstop on every row; first offender is row ",
         bad)
  }

  absorbing <- which(rowSums(!is.na(tmat)) == 0L)

  ## ---- Per-subject contiguity and absorbing-state rules ----
  data <- data[order(data$id, data$Tstart), , drop = FALSE]
  ids <- data$id
  starts <- which(!duplicated(ids))
  stops  <- c(starts[-1L] - 1L, length(ids))

  for (s in seq_along(starts)) {
    rows <- starts[s]:stops[s]
    sub  <- data[rows, , drop = FALSE]
    n    <- nrow(sub)
    sid  <- sub$id[1L]

    # Sstart not absorbing on any row
    if (any(sub$Sstart %in% absorbing)) {
      stop("subject '", sid, "': Sstart is an absorbing state on row ",
           which(sub$Sstart %in% absorbing)[1L], " (this should be ",
           "the previous row's Sstop only)")
    }

    if (n >= 2L) {
      # Temporal contiguity
      gaps <- abs(sub$Tstop[-n] - sub$Tstart[-1L])
      if (any(gaps > tol)) {
        bad <- which(gaps > tol)[1L]
        stop("subject '", sid, "': intervals are not temporally ",
             "contiguous (Tstop[", bad, "] = ", sub$Tstop[bad],
             ", Tstart[", bad + 1L, "] = ", sub$Tstart[bad + 1L], ")")
      }
      # Spatial contiguity
      mismatches <- which(sub$Sstop[-n] != sub$Sstart[-1L])
      if (length(mismatches)) {
        bad <- mismatches[1L]
        stop("subject '", sid, "': states are not spatially contiguous ",
             "(Sstop[", bad, "] = ", sub$Sstop[bad],
             ", Sstart[", bad + 1L, "] = ", sub$Sstart[bad + 1L], ")")
      }
      # No row after one whose Sstop is absorbing
      if (any(sub$Sstop[-n] %in% absorbing)) {
        stop("subject '", sid, "': has a row after entering absorbing ",
             "state ", sub$Sstop[-n][sub$Sstop[-n] %in% absorbing][1L])
      }
    }

    # Censoring (Sstart == Sstop) only on the final row
    self <- which(sub$Sstart == sub$Sstop)
    if (length(self)) {
      if (any(self != n)) {
        stop("subject '", sid, "': Sstart == Sstop is permitted only ",
             "on the last row (censoring); offender is row ",
             self[self != n][1L])
      }
    }

    # Non-censored transitions must be allowed by tmat
    transitioning <- sub$Sstart != sub$Sstop
    if (any(transitioning)) {
      tr_rows <- which(transitioning)
      for (r in tr_rows) {
        if (is.na(tmat[sub$Sstart[r], sub$Sstop[r]])) {
          stop("subject '", sid, "': transition ", sub$Sstart[r],
               " -> ", sub$Sstop[r], " on row ", r,
               " is not allowed by tmat")
        }
      }
    }
  }

  invisible(TRUE)
}


#' Convert Validated Interval Data to Long Multistate Format
#'
#' Expands each row of an interval-format multistate dataset into one
#' row per allowed transition out of the interval's starting state,
#' with \code{status = 1} on the row whose destination matches the
#' actual transition (and \code{0} on the others). The result is the
#' long format expected by \code{\link{fit_chaz}} and other
#' counting-process style routines.
#'
#' @param data A validated interval data frame with columns \code{id},
#'   \code{Tstart}, \code{Tstop}, \code{Sstart}, \code{Sstop}, plus
#'   optional \code{cluster} and \code{group} columns. Run
#'   \code{\link{validate_intervals}()} first to confirm the data
#'   satisfy the package's contiguity and absorbing-state rules.
#' @param tmat A K x K transition matrix from \code{\link{trans_mat}()}.
#'
#' @return A data frame in long multistate format with columns
#'   \code{id}, \code{Tstart}, \code{Tstop}, \code{from}, \code{to},
#'   \code{trans}, \code{status}, plus \code{cluster} and \code{group}
#'   if present in \code{data}.
#'
#' @details
#' Most users do not need to call this function directly --
#' \code{\link{patp}()} runs the full pipeline (parse, validate,
#' expand, fit). It is exported so advanced users can compose the
#' lower-level building blocks (\code{\link{fit_chaz}},
#' \code{\link{prodint_AJ}}, \code{\link{cluster_boot}}) into custom
#' analyses.
#'
#' @examples
#' data(example_msm)
#' tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
#'                   names = c("Healthy", "Ill", "Dead"))
#' long <- intervals_to_long(example_msm, tmat)
#' head(long)
#'
#' @export
intervals_to_long <- function(data, tmat) {

  K <- nrow(tmat)
  pieces <- vector("list", nrow(data))

  for (i in seq_len(nrow(data))) {
    row_i <- data[i, , drop = FALSE]
    h <- row_i$Sstart
    censored <- row_i$Sstart == row_i$Sstop

    out_idx <- which(!is.na(tmat[h, ]))
    if (length(out_idx) == 0L) next  # absorbing -- shouldn't happen post-validation

    block <- data.frame(
      id     = rep(row_i$id,     length(out_idx)),
      Tstart = rep(row_i$Tstart, length(out_idx)),
      Tstop  = rep(row_i$Tstop,  length(out_idx)),
      from   = rep(h,            length(out_idx)),
      to     = out_idx,
      trans  = tmat[h, out_idx],
      status = as.integer(!censored & out_idx == row_i$Sstop)
    )

    if (!is.null(row_i$cluster)) block$cluster <- row_i$cluster
    if (!is.null(row_i$group))   block$group   <- row_i$group

    pieces[[i]] <- block
  }

  out <- do.call(rbind, pieces[!vapply(pieces, is.null, logical(1L))])
  rownames(out) <- NULL
  structure(out, class = c("cmsdata", "data.frame"))
}

make_valid_intervals <- function() {
  # 3 subjects, illness-death (1=H, 2=I, 3=D), no recovery
  # subj 1: H -> I -> D
  # subj 2: H -> D
  # subj 3: H censored
  data.frame(
    id     = c(1, 1, 2, 3),
    Tstart = c(0, 0.5, 0, 0),
    Tstop  = c(0.5, 1.5, 1.0, 2.0),
    Sstart = c(1, 2, 1, 1),
    Sstop  = c(2, 3, 3, 1)
  )
}

test_that("validate_intervals accepts a valid dataset", {
  d <- make_valid_intervals()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_invisible(validate_intervals(d, tmat))
  expect_true(validate_intervals(d, tmat))
})

test_that("validate_intervals errors on missing columns", {
  d <- make_valid_intervals()[, c("id", "Tstart", "Tstop")]
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_error(validate_intervals(d, tmat), "missing required columns")
})

test_that("validate_intervals errors on non-integer states", {
  d <- make_valid_intervals()
  d$Sstart <- d$Sstart + 0.5
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_error(validate_intervals(d, tmat), "integer-valued")
})

test_that("validate_intervals errors on Tstart >= Tstop", {
  d <- make_valid_intervals()
  d$Tstop[1] <- d$Tstart[1]
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_error(validate_intervals(d, tmat), "Tstart must be < Tstop")
})

test_that("validate_intervals errors on out-of-range states", {
  d <- make_valid_intervals()
  d$Sstart[1] <- 99
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_error(validate_intervals(d, tmat), "1\\.\\.3")
})

test_that("validate_intervals errors on temporal gaps within subject", {
  d <- make_valid_intervals()
  d$Tstart[2] <- 0.7        # gap between row 1's Tstop (0.5) and row 2's Tstart
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_error(validate_intervals(d, tmat),
               "not temporally contiguous")
})

test_that("validate_intervals errors on spatial gaps within subject", {
  d <- make_valid_intervals()
  d$Sstart[2] <- 1          # row 1 ends in state 2 but row 2 starts in 1
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_error(validate_intervals(d, tmat),
               "not spatially contiguous")
})

test_that("validate_intervals errors when row follows absorbing entry", {
  # subj 1 enters Dead (3) at row 1 Tstop, then has another row -- illegal
  d <- data.frame(
    id     = c(1, 1),
    Tstart = c(0, 0.5),
    Tstop  = c(0.5, 1.0),
    Sstart = c(1, 3),
    Sstop  = c(3, 3)
  )
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_error(validate_intervals(d, tmat), "absorbing")
})

test_that("validate_intervals errors on disallowed transitions", {
  # tmat does not allow direct 1 -> 3, but data records it
  d <- data.frame(
    id     = 1,
    Tstart = 0,
    Tstop  = 1,
    Sstart = 1,
    Sstop  = 3
  )
  tmat <- trans_mat(list(2, 3, integer(0)))    # only 1->2 and 2->3
  expect_error(validate_intervals(d, tmat),
               "not allowed by tmat")
})

test_that("validate_intervals errors when censoring not on last row", {
  d <- data.frame(
    id     = c(1, 1),
    Tstart = c(0,   0.5),
    Tstop  = c(0.5, 1.0),
    Sstart = c(1,   1),
    Sstop  = c(1,   2)         # row 1 has Sstart == Sstop but is not last
  )
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_error(validate_intervals(d, tmat),
               "Sstart == Sstop is permitted only on the last row")
})

test_that("validate_intervals handles single-row subjects", {
  d <- data.frame(
    id     = c(1, 2),
    Tstart = c(0, 0),
    Tstop  = c(2, 2),
    Sstart = c(1, 1),
    Sstop  = c(1, 2)            # subj 1 censored, subj 2 transitioned
  )
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  expect_true(validate_intervals(d, tmat))
})

test_that("intervals_to_long produces correct long-format output", {
  d <- make_valid_intervals()
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  long <- intervals_to_long(d, tmat)

  expect_s3_class(long, "cmsdata")
  expect_true(all(c("id", "Tstart", "Tstop", "from", "to", "trans",
                    "status") %in% names(long)))

  # Subject 1 has 2 intervals: H sojourn (2 transitions out) +
  # I sojourn (1 transition out) = 3 rows
  expect_equal(sum(long$id == 1), 3)

  # Subject 1's H -> I row (status=1)
  s1_hi <- long[long$id == 1 & long$Tstart == 0 & long$to == 2, ]
  expect_equal(s1_hi$status, 1)

  # Subject 1's H -> D row (status=0; competing risk not realized)
  s1_hd <- long[long$id == 1 & long$Tstart == 0 & long$to == 3, ]
  expect_equal(s1_hd$status, 0)
})

test_that("intervals_to_long correctly handles censoring", {
  # Subject censored healthy at t=2: 1 interval, 2 rows out, both status=0
  d <- data.frame(
    id = 1, Tstart = 0, Tstop = 2, Sstart = 1, Sstop = 1
  )
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  long <- intervals_to_long(d, tmat)

  expect_equal(nrow(long), 2)
  expect_true(all(long$status == 0))
})

test_that("intervals_to_long carries cluster and group columns through", {
  d <- make_valid_intervals()
  d$cluster <- c(10, 10, 20, 30)
  d$group   <- c("A", "A", "B", "B")
  tmat <- trans_mat(list(c(2, 3), 3, integer(0)))
  long <- intervals_to_long(d, tmat)

  expect_true("cluster" %in% names(long))
  expect_true("group" %in% names(long))
  expect_equal(long$cluster[long$id == 1][1], 10)
  expect_equal(long$group[long$id == 1][1], "A")
})

test_that("intervals_to_long handles recovery transitions", {
  # H -> I -> H -> censored
  d <- data.frame(
    id     = c(1, 1, 1),
    Tstart = c(0,   0.5, 1.2),
    Tstop  = c(0.5, 1.2, 2.5),
    Sstart = c(1,   2,   1),
    Sstop  = c(2,   1,   1)
  )
  tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)))
  long <- intervals_to_long(d, tmat)

  # Each interval expands to 2 rows (2 allowed transitions out of each
  # transient state). 3 intervals × 2 = 6 rows.
  expect_equal(nrow(long), 6)
  # H -> I observed at time 0.5
  hi_row <- long[long$Tstart == 0 & long$to == 2, ]
  expect_equal(hi_row$status, 1)
  # I -> H observed at time 1.2
  ih_row <- long[long$Tstart == 0.5 & long$to == 1, ]
  expect_equal(ih_row$status, 1)
  # Final H sojourn is censored: both rows status = 0
  censored_rows <- long[long$Tstart == 1.2, ]
  expect_true(all(censored_rows$status == 0))
})

# Regression test: clusteredMSM's prodint_AJ() must agree with
# mstate::probtrans() on a progressive (acyclic) illness-death model
# to within numerical precision. mstate is in Suggests only, so this
# whole file is skipped when mstate is unavailable.

skip_if_not_installed("mstate")

# ---- Build a small progressive illness-death dataset --------------
# States: 1 = Healthy, 2 = Ill, 3 = Dead. 3 is absorbing.
# Allowed transitions: 1 -> 2, 1 -> 3, 2 -> 3 (no recovery).
# We simulate latent illness, healthy-death, ill-death and censoring
# times once, then materialise BOTH the mstate wide format and the
# clusteredMSM interval format from the *same* realised paths.
make_progressive_data <- function(n = 50, seed = 20260508) {
  set.seed(seed)

  t_ill   <- stats::rexp(n, rate = 0.6)   # latent H -> I
  t_dth_h <- stats::rexp(n, rate = 0.3)   # latent H -> D
  t_dth_i <- stats::rexp(n, rate = 0.5)   # latent I -> D (sojourn in I)
  t_cens  <- stats::runif(n, 1, 5)

  wide <- data.frame(
    id          = seq_len(n),
    illt        = NA_real_,
    illstatus   = 0L,
    deatht      = NA_real_,
    deathstatus = 0L
  )

  intervals <- vector("list", n)

  for (i in seq_len(n)) {
    if (t_ill[i] < t_dth_h[i] && t_ill[i] < t_cens[i]) {
      # H -> I, then either I -> D or censored ill
      ill_t <- t_ill[i]
      wide$illt[i]      <- ill_t
      wide$illstatus[i] <- 1L
      dth_t <- ill_t + t_dth_i[i]
      if (dth_t < t_cens[i]) {
        wide$deatht[i]      <- dth_t
        wide$deathstatus[i] <- 1L
        intervals[[i]] <- data.frame(
          id     = i,
          Tstart = c(0, ill_t),
          Tstop  = c(ill_t, dth_t),
          Sstart = c(1L, 2L),
          Sstop  = c(2L, 3L)
        )
      } else {
        wide$deatht[i]      <- t_cens[i]
        wide$deathstatus[i] <- 0L
        intervals[[i]] <- data.frame(
          id     = i,
          Tstart = c(0, ill_t),
          Tstop  = c(ill_t, t_cens[i]),
          Sstart = c(1L, 2L),
          Sstop  = c(2L, 2L)
        )
      }
    } else if (t_dth_h[i] < t_ill[i] && t_dth_h[i] < t_cens[i]) {
      # H -> D directly
      dth_t <- t_dth_h[i]
      wide$illt[i]        <- dth_t
      wide$illstatus[i]   <- 0L
      wide$deatht[i]      <- dth_t
      wide$deathstatus[i] <- 1L
      intervals[[i]] <- data.frame(
        id = i, Tstart = 0, Tstop = dth_t, Sstart = 1L, Sstop = 3L
      )
    } else {
      # Censored healthy
      wide$illt[i]        <- t_cens[i]
      wide$illstatus[i]   <- 0L
      wide$deatht[i]      <- t_cens[i]
      wide$deathstatus[i] <- 0L
      intervals[[i]] <- data.frame(
        id = i, Tstart = 0, Tstop = t_cens[i], Sstart = 1L, Sstop = 1L
      )
    }
  }

  list(wide = wide, intervals = do.call(rbind, intervals))
}


test_that("prodint_AJ agrees with mstate::probtrans on a progressive model", {
  dat <- make_progressive_data(n = 50)

  # ---- mstate pipeline -------------------------------------------
  tmat_m <- mstate::transMat(list(c(2, 3), 3, integer(0)),
                             names = c("Healthy", "Ill", "Dead"))

  ms_long <- mstate::msprep(
    time   = c(NA, "illt",      "deatht"),
    status = c(NA, "illstatus", "deathstatus"),
    data   = dat$wide,
    trans  = tmat_m,
    id     = "id"
  )

  cox_m <- survival::coxph(
    survival::Surv(Tstart, Tstop, status) ~ survival::strata(trans),
    data = ms_long, method = "breslow"
  )
  msf <- mstate::msfit(cox_m, trans = tmat_m, variance = FALSE)
  pt  <- mstate::probtrans(msf, predt = 0, variance = FALSE)
  # pt[[1]] is P(0, t) starting from state 1 (Healthy)
  ms_curve <- pt[[1]]   # columns: time, pstate1, pstate2, pstate3

  # ---- clusteredMSM pipeline --------------------------------------
  tmat_c <- trans_mat(list(c(2, 3), 3, integer(0)),
                      names = c("Healthy", "Ill", "Dead"))

  validate_intervals(dat$intervals, tmat_c)
  long_c <- intervals_to_long(dat$intervals, tmat_c)
  haz_c  <- fit_chaz(long_c, tmat_c)
  cm_curve <- prodint_AJ(haz_c, tmat_c, predt = 0, h = 1)

  # ---- Align both onto a common grid -----------------------------
  # Use a dense union of jump times so the comparison covers the
  # full support, not just one of the two outputs.
  grid <- sort(unique(c(ms_curve$time, cm_curve$time)))

  diffs_per_state <- vapply(1:3, function(j) {
    col <- paste0("pstate", j)
    # Initial value at t = 0 for column j is 1 if j == 1 (we start in
    # state 1) else 0 -- needed for grid points before the first jump.
    init <- if (j == 1L) 1 else 0
    ms_g <- step_interp(ms_curve$time, ms_curve[[col]], grid, init = init)
    cm_g <- step_interp(cm_curve$time, cm_curve[[col]], grid, init = init)
    max(abs(ms_g - cm_g))
  }, numeric(1))

  max_diff <- max(diffs_per_state)
  if (max_diff > 1e-10) {
    # Locate the worst time point across all three states for diagnostics.
    worst <- vapply(1:3, function(j) {
      col <- paste0("pstate", j)
      init <- if (j == 1L) 1 else 0
      ms_g <- step_interp(ms_curve$time, ms_curve[[col]], grid, init = init)
      cm_g <- step_interp(cm_curve$time, cm_curve[[col]], grid, init = init)
      d <- abs(ms_g - cm_g)
      c(state = j, t = grid[which.max(d)], diff = max(d))
    }, numeric(3))
    message("Per-state max abs differences:\n",
            paste(capture.output(print(t(worst))), collapse = "\n"))
  }

  expect_lt(max_diff, 1e-10)
})

#' Fit Cox Model Stratified by Transition and Return Cumulative Hazards
#'
#' Fits a Cox proportional hazards model stratified by transition type and
#' returns the Breslow-estimated cumulative transition hazards in long
#' format.
#'
#' @param data A data frame in long multistate format with columns
#'   \code{Tstart}, \code{Tstop}, \code{status}, and \code{trans}.
#'   Typically the output of \code{\link{intervals_to_long}()}.
#' @param tmat A K x K transition matrix from \code{trans_mat()}. Used
#'   only to validate that all transition IDs in \code{data} are defined
#'   in \code{tmat}; not used in the fit itself.
#' @param weights Optional numeric vector of length \code{nrow(data)}
#'   with observation weights. Pass \code{1 / data$clust.size} for the
#'   inverse-cluster-size-weighted estimator. \code{NULL} (default)
#'   means unweighted.
#'
#' @return A data frame with columns:
#' \itemize{
#'   \item \code{time}: jump time of the transition.
#'   \item \code{Haz}: Breslow estimate of the cumulative transition
#'     hazard at \code{time}.
#'   \item \code{trans}: integer transition ID (matches \code{tmat}).
#' }
#' Sorted by \code{trans} then \code{time}.
#'
#' @details
#' The model fitted is
#' \code{coxph(Surv(Tstart, Tstop, status) ~ strata(trans),
#' method = "breslow")}.
#' This is the Andersen-Gill counting-process formulation, which handles
#' multiple intervals per subject -- including non-monotone processes
#' with recovery -- provided the input data was correctly constructed
#' (one row per sojourn x candidate transition).
#'
#' @examples
#' data(example_msm)
#' tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
#'                   names = c("Healthy", "Ill", "Dead"))
#'
#' # fit_chaz takes long-format data; intervals_to_long() converts
#' # the interval format users supply. Most users reach fit_chaz
#' # indirectly via patp() -- this example shows the manual pipeline.
#' long <- intervals_to_long(example_msm, tmat)
#' haz  <- fit_chaz(long, tmat)
#' head(haz)
#'
#' @importFrom survival coxph Surv strata basehaz
#' @export
fit_chaz <- function(data, tmat, weights = NULL) {

  required <- c("Tstart", "Tstop", "status", "trans")
  missing_cols <- setdiff(required, names(data))
  if (length(missing_cols)) {
    stop("'data' is missing columns: ", paste(missing_cols, collapse = ", "))
  }

  if (!is.null(weights)) {
    if (!is.numeric(weights) || length(weights) != nrow(data)) {
      stop("'weights' must be a numeric vector of length nrow(data)")
    }
    if (any(weights < 0, na.rm = TRUE)) {
      stop("'weights' must be non-negative")
    }
  }

  # Validate transition IDs against tmat
  if (!missing(tmat) && !is.null(tmat)) {
    valid_ids <- sort(unique(as.vector(tmat[!is.na(tmat)])))
    bad <- setdiff(unique(data$trans), valid_ids)
    if (length(bad)) {
      stop("data$trans contains IDs not in tmat: ",
           paste(bad, collapse = ", "))
    }
  }

  fit <- if (is.null(weights)) {
    survival::coxph(
      survival::Surv(Tstart, Tstop, status) ~ survival::strata(trans),
      data   = data,
      method = "breslow"
    )
  } else {
    survival::coxph(
      survival::Surv(Tstart, Tstop, status) ~ survival::strata(trans),
      data    = data,
      weights = weights,
      method  = "breslow"
    )
  }

  bh <- survival::basehaz(fit, centered = FALSE)

  # survival::basehaz formats strata levels as "trans=1", "trans=2", ...
  # in modern releases; older releases used plain "1", "2", .... Detect
  # which is in use and parse accordingly (avoids a coercion warning
  # from running the wrong cast first).
  raw <- as.character(bh$strata)
  trans <- if (any(grepl("=", raw, fixed = TRUE))) {
    as.integer(sub(".*=", "", raw))
  } else {
    as.integer(raw)
  }

  out <- data.frame(
    time  = bh$time,
    Haz   = bh$hazard,
    trans = trans
  )

  out <- out[order(out$trans, out$time), ]
  rownames(out) <- NULL
  out
}

#' Population-Averaged Transition Probabilities for Multistate Process Data
#'
#' Computes the working-independence Aalen-Johansen estimator of the
#' transition probability \eqn{P(X(t) = j \mid X(s) = h)} for clustered or
#' independent multistate process data, with cluster-bootstrap
#' standard errors, pointwise confidence intervals, and (optionally)
#' simultaneous confidence bands. If a grouping variable is supplied
#' on the right-hand side of the formula, also conducts a two-sample
#' Kolmogorov-Smirnov-type test of curve equality.
#'
#' @param formula A formula of the form
#'   \code{msm(Tstart, Tstop, Sstart, Sstop) ~ 1} for a one-sample
#'   analysis, or \code{msm(Tstart, Tstop, Sstart, Sstop) ~ group} for
#'   a two-sample analysis (estimate + test).
#' @param data A data frame containing the variables in \code{formula}
#'   plus the \code{id} (and optional \code{cluster}) columns.
#' @param tmat A K x K transition matrix, typically built with
#'   \code{\link{trans_mat}()}.
#' @param id Character. Name of the subject ID column. Required.
#' @param cluster Character or \code{NA}. Name of the cluster ID column.
#'   When \code{NA} (default), the bootstrap resamples individuals;
#'   when supplied, it resamples whole clusters.
#' @param h Integer in 1..K. Starting state.
#' @param j Integer in 1..K. Ending state.
#' @param s Numeric scalar. Conditioning time. Default 0.
#' @param weighted Logical. Inverse-cluster-size weighting to account for 
#'  potentially informative cluster size (requires \code{cluster}). 
#'  Default \code{FALSE}.
#' @param LMAJ Logical. Use the landmark Aalen-Johansen estimator
#'   (recommended when \code{s > 0} and the Markov assumption is
#'   questionable). Default \code{FALSE}.
#' @param B Integer. Number of bootstrap replications. \code{B = 0}
#'   skips inference and is permitted only for one-sample formulas.
#'   Default 1000.
#' @param cband Logical. If \code{TRUE}, return simultaneous
#'   confidence band limits at the level set by \code{level}
#'   (recommended \code{B >= 1000}).
#' @param design Character, two-sample only. One of
#'   \code{"auto"} (default), \code{"shared"},
#'   \code{"cluster_random"}, \code{"indep_random"}. Selects the
#'   two-sample regime; see the \emph{Two-sample designs} section.
#'   \code{"auto"} infers the regime from the cluster/group
#'   structure of the data, defaulting to \code{"indep_random"} when
#'   each cluster carries a single group (a safer default than
#'   assuming cluster randomization). When that fallback fires, a warning
#'   is emitted asking the user to set \code{"cluster_random"}
#'   explicitly if the per-group cluster counts were fixed by cluster
#'   randomization.
#' @param level Confidence level. Default 0.95.
#' @param seed Optional integer for reproducible bootstrap.
#'
#' @return An S3 object of class \code{patp} containing:
#' \itemize{
#'   \item \code{call}, \code{formula}: the original call and formula.
#'   \item \code{curves}: a data frame with columns \code{time},
#'     \code{P}, \code{group} (if two-sample), and (if \code{B > 0})
#'     \code{se}, \code{ll}, \code{ul} -- and \code{ll.band},
#'     \code{ul.band} if \code{cband = TRUE}.
#'   \item \code{test}: \code{NULL} (one-sample) or a list with the
#'     observed K-S statistic and bootstrap p-value (two-sample).
#'   \item \code{n_subjects}, \code{n_clusters}, \code{groups},
#'     \code{h}, \code{j}, \code{s}, \code{B}.
#' }
#'
#' @section Two-sample designs:
#' Three regimes are supported, each with its own bootstrap scheme
#' and asymptotic scaling factor; the test statistic in every case is
#' \eqn{T = c_n \sup_t |\hat P_1(t) - \hat P_0(t)|}.
#' \describe{
#'   \item{\code{"shared"} (case i): dependent groups (multicenter
#'     trial).}{Every cluster contributes observations from both
#'     groups -- e.g., each hospital randomizes its patients to
#'     treatment or control. Unstratified cluster bootstrap;
#'     \eqn{c_n = \sqrt{n}} (Bakoyannis 2021, Theorem 3), with
#'     \eqn{n} the total number of clusters.}
#'   \item{\code{"cluster_random"} (case ii.a): cluster-randomized
#'     trial.}{Each cluster is assigned to exactly one group, with
#'     per-group cluster counts \eqn{n_1, n_2} fixed by the
#'     randomization. Stratified cluster bootstrap (resample within
#'     each group); \eqn{c_n = \sqrt{n_1 n_2 / (n_1 + n_2)}}
#'     (Bakoyannis & Bandyopadhyay 2022, Theorem 2). This regime
#'     must be opted into; \code{"auto"} will not select it.}
#'   \item{\code{"indep_random"} (case ii.b): independent
#'     observational comparison.}{Each cluster carries one group,
#'     but \eqn{n_1, n_2} are random (the population, not the
#'     analyst, decided who landed in which group). Unstratified
#'     cluster bootstrap; \eqn{c_n = \sqrt{n_1 n_2 / (n_1 + n_2)}}.
#'     The asymptotic regime is the same two-independent-samples
#'     limit as \code{"cluster_random"}; only the bootstrap
#'     differs. This is the \code{"auto"} default when each cluster
#'     carries a single group.}
#' }
#' Mixed structures (some clusters carry both groups, some only one)
#' are not supported in this version.
#'
#' \code{patp()} validates the supplied data against the requested
#' design and stops with an informative error if they disagree.
#'
#' The K-S statistic in v0.1 uses unit weight at every time point.
#' Weighted variants -- following the harmonic-mean weight
#' \eqn{W(t) = \prod_p Y_p(t) / \sum_p Y_p(t)} of Bakoyannis (2021)
#' Section 2.5 and the related construction in Bakoyannis & Bandyopadhyay
#' (2022) -- are planned for v0.2.
#'
#' @section Standard errors and confidence intervals:
#' The cluster bootstrap is run \emph{once}, on the original
#' probability scale, in line with Bakoyannis (2021) Theorem 2. The
#' \code{se} column in \code{curves} is the bootstrap standard
#' deviation of \eqn{\hat P(t)} on that scale, i.e.
#' \code{apply(boot_matrix, 1, sd, na.rm = TRUE)}.
#'
#' Pointwise CIs (\code{ll}, \code{ul}) are then built on the cloglog
#' scale \eqn{g(p) = \log(-\log p)} using the delta-method
#' standardization \eqn{\mathrm{SE}_g(t) = \mathrm{SE}(\hat P(t)) /
#' |\hat P(t) \log \hat P(t)|}, and back-transformed via
#' \eqn{p = \exp(-\exp(\cdot))}. Simultaneous bands
#' (\code{ll.band}, \code{ul.band}) use the same \eqn{\mathrm{SE}_g};
#' the \code{level}-quantile of the supremum gives the critical value.
#'
#' Because the cloglog transformation is nonlinear, \code{se} and the
#' resulting CI widths are \emph{not} equal in general -- the
#' CI is asymmetric on the probability scale. Report \code{se} for
#' descriptive purposes; use \code{ll}/\code{ul} for inference.
#'
#' The simultaneous band is trimmed to the central 90\% of observed
#' jump times by default to avoid the band fanning out near the
#' extremes of follow-up; outside that window \code{ll.band} and
#' \code{ul.band} are \code{NA}. See \code{\link{confidence_band}()}
#' for the \code{trim} argument.
#'
#' Note: in this version the simultaneous band uses an
#' equal-precision-type construction on the cloglog scale, which is
#' asymptotically valid but differs from the Hall-Wellner-type
#' construction of Bakoyannis (2021) Section 2.3 (planned for a
#' future release). See \code{\link{confidence_band}()} for the full
#' disclosure.
#'
#' @references
#' Bakoyannis, G. (2021). Nonparametric analysis of nonhomogeneous
#' multistate processes with clustered observations.
#' \emph{Biometrics}, 77(2), 533-546. \doi{10.1111/biom.13327}
#'
#' Bakoyannis, G., & Bandyopadhyay, D. (2022). Nonparametric tests
#' for multistate processes with clustered data. \emph{Annals of the
#' Institute of Statistical Mathematics}, 74(5), 837-867.
#' \doi{10.1007/s10463-021-00819-x}
#' 
#' Putter H, Spitoni C (2018). Non-parametric estimation of transition
#' probabilities in non-Markov multi-state models: the landmark
#' Aalen-Johansen estimator. \emph{Statistical Methods in Medical
#' Research} 27(7):2081-2092. \doi{10.1177/0962280216674497}
#'
#' @examples
#' data(example_msm)
#' tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
#'                   names = c("Healthy", "Ill", "Dead"))
#'
#' # One-sample: P(Ill at t | Healthy at 0). B is small here for speed;
#' # use B >= 1000 for reported results.
#' fit <- patp(msm(Tstart, Tstop, Sstart, Sstop) ~ 1,
#'             data = example_msm, tmat = tmat,
#'             id = "id", cluster = "cluster",
#'             h = 1, j = 2, s = 0,
#'             B = 50, seed = 1)
#' fit
#'
#' # Two-sample (estimate + test in one call). Each cluster in
#' # example_msm carries both treatment levels, so design = "shared".
#' tt <- patp(msm(Tstart, Tstop, Sstart, Sstop) ~ treatment,
#'            data = example_msm, tmat = tmat,
#'            id = "id", cluster = "cluster",
#'            h = 1, j = 2, B = 50,
#'            design = "shared", seed = 1)
#' tt
#'
#' @seealso \code{\link{msm}}, \code{\link{trans_mat}},
#'   \code{\link{validate_intervals}}.
#' @export
patp <- function(formula, data, tmat,
                 id, cluster = NA,
                 h, j, s = 0,
                 weighted = FALSE, LMAJ = FALSE,
                 B = 1000, cband = FALSE,
                 design = c("auto", "shared",
                            "cluster_random", "indep_random"),
                 level = 0.95, seed = NULL) {

  design <- match.arg(design)

  call_ <- match.call()

  ## ---- Parse and validate ----
  parsed <- parse_msm_formula(formula, data, id = id, cluster = cluster)
  validate_intervals(parsed$data, tmat)
  long <- intervals_to_long(parsed$data, tmat)

  if (weighted && !parsed$has_cluster) {
    stop("weighted = TRUE requires the 'cluster' argument")
  }

  ## ---- Dispatch based on RHS ----
  is_two_sample <- !is.null(parsed$group)
  if (is_two_sample && B == 0L) {
    stop("two-sample formulas require B > 0")
  }

  if (!is_two_sample && design != "auto") {
    stop("'design' applies only to two-sample formulas")
  }

  fit <- if (is_two_sample) {
    .patp_twosample(long, tmat,
                    has_cluster = parsed$has_cluster,
                    h = h, j = j, s = s,
                    weighted = weighted, LMAJ = LMAJ,
                    B = B, cband = cband, level = level, seed = seed,
                    group_name = parsed$group_name,
                    design = design)
  } else {
    .patp_onesample(long, tmat,
                    has_cluster = parsed$has_cluster,
                    h = h, j = j, s = s,
                    weighted = weighted, LMAJ = LMAJ,
                    B = B, cband = cband, level = level, seed = seed)
  }

  fit$call    <- call_
  fit$formula <- formula
  fit$h <- h; fit$j <- j; fit$s <- s; fit$B <- B
  fit$n_subjects <- length(unique(long$id))

  structure(fit, class = c("patp", "list"))
}

#' Truncate a Long-Format Multistate Dataset at a Landmark Time
#'
#' Restricts a long-format multistate dataset to follow-up after a
#' landmark time \code{s}. Intervals ending before \code{s} are removed;
#' intervals straddling \code{s} are truncated to start at \code{s}.
#'
#' @param data A long-format data frame, typically produced by
#'   \code{\link{intervals_to_long}()}, with columns \code{Tstart},
#'   \code{Tstop}, \code{from}, \code{to}, \code{trans}, \code{status},
#'   \code{id}, and (optionally) \code{cluster}.
#' @param s Numeric scalar. The landmark time.
#'
#' @return A data frame of the same shape as \code{data}, restricted and
#'   truncated as described above.
#'
#' @details
#' This function is used in the landmark Aalen-Johansen estimator, which
#' relaxes the Markov assumption by re-fitting hazards on the cohort
#' still under observation at time \code{s}.
#'
#' Important: this function only truncates the time axis. To compute the
#' landmark estimator, you must also restrict to subjects who are in the
#' relevant starting state at time \code{s} -- use \code{state_at()} for
#' that step.
#'
#' Intervals where \code{Tstop == s} exactly are dropped (the subject has
#' already transitioned by \code{s}); intervals where \code{Tstart >= s}
#' are kept unchanged; intervals straddling \code{s}
#' (\code{Tstart < s < Tstop}) have their \code{Tstart} reset to \code{s}.
#' If a straddling interval was going to end in an event
#' (\code{status == 1}) at \code{Tstop > s}, the event is preserved.
#'
#' @references
#' Putter H, Spitoni C (2018). Non-parametric estimation of transition
#' probabilities in non-Markov multi-state models: the landmark
#' Aalen-Johansen estimator. \emph{Statistical Methods in Medical
#' Research} 27(7):2081-2092. \doi{10.1177/0962280216674497}
#'
#' Bakoyannis G (2021). Nonparametric analysis of nonhomogeneous
#' multistate processes with clustered observations.
#' \emph{Biometrics} 77(2):533-546. \doi{10.1111/biom.13327}
#'
#' @examples
#' data(example_msm)
#' tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
#'                   names = c("Healthy", "Ill", "Dead"))
#' long <- intervals_to_long(example_msm, tmat)
#'
#' # Restrict follow-up to after the landmark time s = 1.5
#' long_lm <- cut_at_lm(long, s = 1.5)
#' head(long_lm)
#'
#' @export
cut_at_lm <- function(data, s) {

  required <- c("Tstart", "Tstop", "status")
  missing_cols <- setdiff(required, names(data))
  if (length(missing_cols)) {
    stop("'data' is missing columns: ", paste(missing_cols, collapse = ", "))
  }
  if (!is.numeric(s) || length(s) != 1L) {
    stop("'s' must be a numeric scalar")
  }

  # Drop intervals that end at or before s
  data <- data[data$Tstop > s, , drop = FALSE]

  # Truncate intervals that straddle s
  straddling <- data$Tstart < s
  data$Tstart[straddling] <- s

  rownames(data) <- NULL
  data
}

test_that("cut_at_lm drops intervals ending before s", {
  d <- data.frame(Tstart = c(0, 0, 1), Tstop = c(0.5, 1.0, 2.5),
                  status = c(0, 1, 0), from = 1, to = 2, trans = 1)
  out <- cut_at_lm(d, s = 1.5)
  expect_equal(nrow(out), 1)
  expect_equal(out$Tstop, 2.5)
})

test_that("cut_at_lm truncates straddling intervals at s", {
  d <- data.frame(Tstart = c(0, 2),  Tstop = c(3, 4),
                  status = c(1, 0),  from = 1, to = 2, trans = 1)
  out <- cut_at_lm(d, s = 1.5)
  expect_equal(out$Tstart, c(1.5, 2))
  expect_equal(out$Tstop,  c(3,   4))
  expect_equal(out$status, c(1,   0))
})

test_that("cut_at_lm leaves intervals starting at or after s alone", {
  d <- data.frame(Tstart = c(2, 3), Tstop = c(2.5, 4),
                  status = c(0, 1), from = 1, to = 2, trans = 1)
  out <- cut_at_lm(d, s = 1.5)
  expect_equal(out$Tstart, c(2, 3))
  expect_equal(out$Tstop,  c(2.5, 4))
})

test_that("cut_at_lm with s <= min(Tstart) returns input unchanged", {
  d <- data.frame(Tstart = c(0, 1), Tstop = c(2, 3),
                  status = c(0, 1), from = 1, to = 2, trans = 1)
  out <- cut_at_lm(d, s = 0)
  expect_equal(nrow(out), 2)
  expect_equal(out$Tstart, c(0, 1))
})

test_that("cut_at_lm with s past all Tstop returns empty", {
  d <- data.frame(Tstart = c(0, 1), Tstop = c(2, 3),
                  status = c(0, 1), from = 1, to = 2, trans = 1)
  out <- cut_at_lm(d, s = 10)
  expect_equal(nrow(out), 0)
})

test_that("cut_at_lm validates input", {
  expect_error(cut_at_lm(data.frame(x = 1), s = 0), "missing columns")
  expect_error(cut_at_lm(data.frame(Tstart=0, Tstop=1, status=0),
                         s = c(1, 2)),
               "numeric scalar")
})

test_that("cut_at_lm preserves all other columns", {
  d <- data.frame(Tstart = 0, Tstop = 2, status = 1,
                  from = 1, to = 2, trans = 1, group = "A",
                  cid = 7, id = 42, stringsAsFactors = FALSE)
  out <- cut_at_lm(d, s = 1)
  expect_equal(names(out), names(d))
  expect_equal(out$group, "A")
  expect_equal(out$cid, 7)
})

#' Generic Cluster Bootstrap Engine
#'
#' Performs the nonparametric cluster bootstrap: resamples whole clusters
#' with replacement and applies a user-supplied statistic function to
#' each bootstrap replicate.
#'
#' @param data A data frame containing the cluster ID column named by
#'   \code{cluster}. All other columns are passed through unchanged to
#'   \code{fn}.
#' @param cluster Character. Name of the cluster ID column.
#' @param B Integer. Number of bootstrap replications.
#' @param fn Function. Called as \code{fn(boot_data, ...)} once per
#'   bootstrap replicate. Must return a numeric vector of fixed length
#'   (the length must be the same across replicates -- typically achieved
#'   by evaluating the statistic on a fixed grid of times).
#' @param ... Additional arguments passed to \code{fn}.
#' @param strata Optional named vector mapping each unique cluster ID
#'   (as character) to a stratum label. When supplied, resampling is
#'   stratified: in each replicate, the original number of clusters in
#'   each stratum is drawn with replacement from the clusters in that
#'   stratum (so per-stratum cluster counts are preserved across
#'   replicates). When \code{NULL} (default), unstratified resampling
#'   is used.
#' @param seed Optional integer. If non-NULL, \code{set.seed(seed)} is
#'   called before bootstrapping for reproducibility.
#' @param verbose Logical. If \code{TRUE}, prints a progress message
#'   every 100 replicates. Default \code{FALSE}.
#'
#' @return A numeric matrix with \code{length(fn(data, ...))} rows and
#'   \code{B} columns. Column \code{b} is the result of \code{fn} applied
#'   to bootstrap replicate \code{b}.
#'
#' @details
#' The bootstrap is nonparametric and at the cluster level: in each
#' replicate, n clusters are sampled with replacement (where n is the
#' number of unique values of \code{data[[cluster]]}), and the resulting
#' cluster-bound rows are stacked into a new data frame.
#'
#' Resampled clusters are re-IDed (1, 2, ..., n) so that downstream
#' code treating cluster IDs as distinct labels (e.g., \code{tapply}
#' aggregations) works correctly even when the same original cluster
#' appears multiple times in a replicate.
#'
#' Stratified resampling (\code{strata}) supports cluster-randomized
#' designs in which each cluster belongs to a single group (case ii of
#' Bakoyannis & Bandyopadhyay 2022): the per-stratum cluster counts
#' are fixed across replicates, so a replicate can never wipe out one
#' of the groups.
#'
#' The bootstrap is statistic-agnostic: the same engine is used for
#' point-estimator standard errors, two-sample tests, and confidence
#' bands. Whatever \code{fn} returns is what gets bootstrapped.
#'
#' @examples
#' data(example_msm)
#'
#' # Cluster-bootstrap a simple summary: the mean of Tstop across
#' # rows. The user-supplied fn can return any fixed-length numeric
#' # vector; the typical use case is a transition probability curve
#' # evaluated on a fixed time grid (which is exactly what patp() does
#' # under the hood).
#' boot <- cluster_boot(
#'   data = example_msm, cluster = "cluster", B = 50,
#'   fn   = function(d) mean(d$Tstop),
#'   seed = 1
#' )
#' c(point = mean(boot, na.rm = TRUE),
#'   se    = stats::sd(boot, na.rm = TRUE))
#'
#' @export
cluster_boot <- function(data, cluster, B, fn, ...,
                         strata = NULL, seed = NULL, verbose = FALSE) {

  if (!is.data.frame(data)) stop("'data' must be a data frame")
  if (!cluster %in% names(data)) stop("column '", cluster, "' not found in data")
  if (!is.numeric(B) || length(B) != 1L || B < 1L) {
    stop("'B' must be a positive integer")
  }
  if (!is.function(fn)) stop("'fn' must be a function")

  if (!is.null(seed)) set.seed(seed)

  cluster_ids <- unique(data[[cluster]])
  n <- length(cluster_ids)

  ## Build the per-replicate cluster sampler. Two modes:
  ##   - unstratified: sample n clusters with replacement from all clusters
  ##   - stratified:   for each stratum, sample its cluster count with
  ##                   replacement from the clusters in that stratum
  if (is.null(strata)) {
    sample_clusters <- function() sample(cluster_ids, n, replace = TRUE)
  } else {
    if (is.null(names(strata))) {
      stop("'strata' must be a named vector (names = cluster IDs)")
    }
    cluster_chr <- as.character(cluster_ids)
    missing_in_strata <- setdiff(cluster_chr, names(strata))
    if (length(missing_in_strata) > 0L) {
      stop("'strata' is missing entries for cluster IDs: ",
           paste(utils::head(missing_in_strata, 5L), collapse = ", "))
    }
    strata_aligned <- unname(strata[cluster_chr])
    by_stratum <- split(cluster_ids, strata_aligned)
    sample_clusters <- function() {
      drawn <- lapply(by_stratum, function(ids)
        sample(ids, length(ids), replace = TRUE))
      unlist(drawn, use.names = FALSE)
    }
  }

  # Pre-split data by cluster for fast subsetting in the loop
  cluster_rows <- split(seq_len(nrow(data)), data[[cluster]])

  # Probe call: try the original data first to determine out_length and
  # validate that fn returns numeric. Tolerate failure -- some fn may
  # only succeed on resamples; in that case defer length determination
  # to the first successful replicate.
  probe <- tryCatch(fn(data, ...), error = function(e) NULL)
  out <- NULL
  out_length <- NA_integer_
  if (!is.null(probe)) {
    if (!is.numeric(probe)) {
      stop("'fn' must return a numeric vector")
    }
    out_length <- length(probe)
    out <- matrix(NA_real_, nrow = out_length, ncol = B)
  }

  for (b in seq_len(B)) {
    drawn <- sample_clusters()
    n_drawn <- length(drawn)

    # Build the bootstrap data frame. Re-ID clusters 1..n_drawn so duplicates
    # are kept distinct for downstream cluster-aware operations.
    pieces <- vector("list", n_drawn)
    for (k in seq_len(n_drawn)) {
      rows <- cluster_rows[[as.character(drawn[k])]]
      d <- data[rows, , drop = FALSE]
      d[[cluster]] <- k
      pieces[[k]] <- d
    }
    boot_data <- do.call(rbind, pieces)

    result <- tryCatch(fn(boot_data, ...), error = function(e) NULL)
    if (is.null(result)) next
    if (!is.numeric(result)) {
      if (is.null(out)) stop("'fn' must return a numeric vector")
      next
    }
    if (is.null(out)) {
      out_length <- length(result)
      out <- matrix(NA_real_, nrow = out_length, ncol = B)
    }
    if (length(result) != out_length) next
    out[, b] <- result

    if (verbose && b %% 100 == 0) {
      message("cluster_boot: completed ", b, " of ", B, " replicates")
    }
  }

  if (is.null(out)) {
    stop("'fn' failed on every bootstrap replicate and on the probe call")
  }

  out
}

# Helper: 6 clusters of varying size, simple data
make_clustered <- function() {
  data.frame(
    cluster = rep(1:6, times = c(3, 2, 4, 1, 3, 2)),
    x   = rnorm(15)
  )
}

test_that("cluster_boot returns matrix of correct shape", {
  set.seed(1)
  data <- make_clustered()
  out <- cluster_boot(data, cluster = "cluster", B = 50,
                      fn = function(d) c(mean(d$x), sd(d$x)),
                      seed = 1)

  expect_true(is.matrix(out))
  expect_equal(dim(out), c(2, 50))
})

test_that("cluster_boot produces variability across replicates", {
  set.seed(1)
  data <- make_clustered()
  out <- cluster_boot(data, cluster = "cluster", B = 100,
                      fn = function(d) mean(d$x), seed = 1)

  expect_true(sd(out[1, ]) > 0)
})

test_that("cluster_boot is reproducible with the same seed", {
  data <- make_clustered()

  out1 <- cluster_boot(data, "cluster", B = 20,
                       fn = function(d) mean(d$x), seed = 42)
  out2 <- cluster_boot(data, "cluster", B = 20,
                       fn = function(d) mean(d$x), seed = 42)

  expect_equal(out1, out2)
})

test_that("cluster_boot resamples whole clusters (not rows)", {
  data <- make_clustered()
  # Each cluster has a unique x mean. After bootstrap, each replicate's
  # set of cluster means should be drawn (with replacement) from the
  # original 6 cluster means.
  cluster_means <- tapply(data$x, data$cluster, mean)

  set.seed(7)
  out <- cluster_boot(
    data, "cluster", B = 200,
    fn = function(d) {
      means <- tapply(d$x, d$cluster, mean)
      mean(means)
    },
    seed = 7
  )

  # The bootstrap replicate mean should be in the convex hull of the
  # original cluster means (since it is an average of values drawn
  # from them).
  expect_true(all(out >= min(cluster_means) - 1e-10))
  expect_true(all(out <= max(cluster_means) + 1e-10))
})

test_that("cluster_boot re-IDs clusters in each replicate", {
  data <- make_clustered()

  # The fn checks the number of unique cluster IDs in the replicate;
  # after resampling with replacement and re-IDing, this should
  # always equal the original number of clusters (n).
  set.seed(99)
  out <- cluster_boot(
    data, "cluster", B = 30,
    fn = function(d) length(unique(d$cluster)),
    seed = 99
  )
  expect_true(all(out == length(unique(data$cluster))))
})

test_that("cluster_boot validates input", {
  data <- make_clustered()

  expect_error(cluster_boot("not a df", "cluster", 10,
                            fn = function(d) 1),
               "data frame")
  expect_error(cluster_boot(data, "missing", 10,
                            fn = function(d) 1),
               "not found")
  expect_error(cluster_boot(data, "cluster", 0,
                            fn = function(d) 1),
               "positive integer")
  expect_error(cluster_boot(data, "cluster", 10, fn = "not a fn"),
               "function")
  expect_error(cluster_boot(data, "cluster", 10,
                            fn = function(d) "not numeric"),
               "numeric vector")
})

test_that("cluster_boot stratified resampling preserves stratum sizes", {
  # 10 clusters, 6 in stratum "A", 4 in stratum "B"
  data <- data.frame(
    cluster    = rep(1:10, each = 3),
    stratum = rep(c(rep("A", 6), rep("B", 4)), each = 3),
    x      = stats::rnorm(30)
  )
  cluster_to_stratum <- tapply(data$stratum, data$cluster,
                               function(x) as.character(x[1L]))
  strata_vec <- as.character(cluster_to_stratum)
  names(strata_vec) <- names(cluster_to_stratum)

  # fn returns the per-replicate counts of the two strata in the resampled
  # data. Each stratum's count (in subjects) should be exactly preserved.
  out <- cluster_boot(
    data, "cluster", B = 50,
    fn = function(d) c(sum(d$stratum == "A"), sum(d$stratum == "B")),
    strata = strata_vec, seed = 11
  )

  # Original counts: 6 clusters x 3 rows = 18 in A; 4 x 3 = 12 in B.
  expect_true(all(out[1, ] == 18))
  expect_true(all(out[2, ] == 12))
})

test_that("cluster_boot stratified samples within stratum only", {
  # 8 clusters split across two strata. Stratum A has clusters 1..4 with
  # mean x = 0; stratum B has clusters 5..8 with mean x = 10. After
  # stratified resampling, every replicate must contain at least one
  # cluster from each stratum.
  data <- data.frame(
    cluster     = rep(1:8, each = 5),
    stratum = rep(c(rep("A", 4), rep("B", 4)), each = 5),
    x       = c(stats::rnorm(20, mean = 0),
                stats::rnorm(20, mean = 10))
  )
  cluster_to_stratum <- tapply(data$stratum, data$cluster,
                               function(x) as.character(x[1L]))
  strata_vec <- as.character(cluster_to_stratum)
  names(strata_vec) <- names(cluster_to_stratum)

  out <- cluster_boot(
    data, "cluster", B = 100,
    fn = function(d) {
      as.numeric(c(any(d$stratum == "A"), any(d$stratum == "B")))
    },
    strata = strata_vec, seed = 7
  )
  expect_true(all(out == 1))
})

test_that("cluster_boot tolerates pathological replicates", {
  data <- make_clustered()

  # fn that errors on certain inputs -- should not crash the boot,
  # should leave NAs in those columns
  set.seed(2)
  out <- cluster_boot(
    data, "cluster", B = 20,
    fn = function(d) {
      if (runif(1) < 0.3) stop("simulated failure")
      mean(d$x)
    },
    seed = 2
  )

  expect_equal(dim(out), c(1, 20))
  expect_true(any(is.na(out)) || all(!is.na(out)))  # at least defined
})

#' Synthetic Clustered Illness-Death-with-Recovery Data
#'
#' A synthetic interval-format multistate dataset used in package
#' examples, the README, and the vignette. The data are generated from
#' an illness-death-with-recovery process with cluster-level frailty,
#' so they exercise the package's headline feature: support for
#' non-monotone (cyclic) transitions.
#'
#' @format A data frame with 81 rows (one row per mutually-exclusive
#'   time interval per subject) and 7 columns:
#' \describe{
#'   \item{id}{Integer subject identifier (40 subjects, ids 1..40).}
#'   \item{cluster}{Integer cluster identifier (8 clusters, 5 subjects
#'     each).}
#'   \item{treatment}{Binary treatment indicator (0 or 1), balanced
#'     20/20 across subjects. Treatment 1 has a higher illness hazard
#'     than treatment 0.}
#'   \item{Tstart}{Numeric start time of the interval.}
#'   \item{Tstop}{Numeric end time of the interval.}
#'   \item{Sstart}{Integer state during the interval: 1 = Healthy,
#'     2 = Ill, 3 = Dead (absorbing).}
#'   \item{Sstop}{Integer state at \code{Tstop}, or equal to
#'     \code{Sstart} on a final, censored row.}
#' }
#'
#' @details
#' The transition structure is
#' \code{trans_mat(list(c(2, 3), c(1, 3), integer(0)))}, i.e.
#' Healthy <-> Ill, Healthy -> Dead, Ill -> Dead. Within each subject
#' rows are temporally and spatially contiguous; censoring is encoded
#' as \code{Sstart == Sstop} on the final row.
#'
#' The same data are also shipped as a CSV at
#' \code{system.file("extdata", "example_data.csv", package = "clusteredMSM")}
#' for users who prefer to mimic loading their own file.
#'
#' The generation script lives in \code{data-raw/example_msm.R} and is
#' fully reproducible (\code{set.seed(2026)}).
#'
#' @source Simulated. See \code{data-raw/example_msm.R} in the package
#'   sources.
#'
#' @examples
#' data(example_msm)
#' head(example_msm)
#' table(example_msm$Sstart, example_msm$Sstop)
"example_msm"

#' Identify Each Subject's State at a Given Time
#'
#' Given a long-format multistate dataset, returns the state each subject
#' occupies at time \code{s}. Used by the landmark Aalen-Johansen estimator
#' to identify the risk set at a conditioning time.
#'
#' @param data A data frame in long format with columns \code{Tstart},
#'   \code{Tstop}, \code{from}, \code{to}, \code{status}, and the subject
#'   ID column named by \code{id}.
#' @param s Numeric scalar. The time at which to evaluate states.
#' @param id Character. Name of the subject ID column. Default \code{"id"}.
#'
#' @return A data frame with columns \code{<id>} and \code{state}, one row
#'   per subject who is under observation at time \code{s}. Subjects whose
#'   follow-up has not yet started (\code{min(Tstart) > s}) or who have
#'   already entered an absorbing state (\code{max(Tstop) <= s} with
#'   \code{status == 1}) are excluded.
#'
#' @details
#' A subject's state at time \code{s} is the \code{from} state of the
#' interval that contains \code{s} (i.e., \code{Tstart <= s < Tstop}).
#' Because the long format duplicates intervals across competing
#' transitions, the result is deduplicated to one row per subject.
#'
#' For exact ties (\code{s == Tstop}), the subject is treated as having
#' transitioned: they appear in their new state if a subsequent interval
#' exists, otherwise they are excluded as absorbed/censored.
#'
#' @references
#' Putter H, Spitoni C (2018). Non-parametric estimation of transition
#' probabilities in non-Markov multi-state models: the landmark
#' Aalen-Johansen estimator. \emph{Statistical Methods in Medical
#' Research} 27(7):2081-2092. \doi{10.1177/0962280216674497}
#' 
#' @examples
#' data(example_msm)
#' tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
#'                   names = c("Healthy", "Ill", "Dead"))
#' long <- intervals_to_long(example_msm, tmat)
#'
#' # Each subject's state at the landmark time s = 1.5
#' head(state_at(long, s = 1.5, id = "id"))
#'
#' @export
state_at <- function(data, s, id = "id") {

  required <- c("Tstart", "Tstop", "from", "to", "status", id)
  missing_cols <- setdiff(required, names(data))
  if (length(missing_cols)) {
    stop("'data' is missing columns: ", paste(missing_cols, collapse = ", "))
  }
  if (!is.numeric(s) || length(s) != 1L) {
    stop("'s' must be a numeric scalar")
  }

  data <- data[order(data[[id]], data$Tstart, data$Tstop), ]

  # Subjects whose follow-up hasn't reached time s
  starts  <- tapply(data$Tstart, data[[id]], min)
  stops   <- tapply(data$Tstop,  data[[id]], max)
  in_obs_window <- names(starts)[starts <= s & stops > s]

  # Soft "absorbed" rule: keep subjects with any interval extending past s.
  has_later <- unique(data[[id]][data$Tstop > s])
  keep_ids <- intersect(in_obs_window, as.character(has_later))

  sub <- data[as.character(data[[id]]) %in% as.character(keep_ids) &
              data$Tstart <= s & data$Tstop > s,
              c(id, "from")]

  out <- unique(sub)
  names(out) <- c(id, "state")
  rownames(out) <- NULL
  out
}

# Helper to construct a small msdata-style example
make_msd <- function() {
  # 4 subjects, illness-death model, transitions 1=H->I, 2=H->D, 3=I->D
  data.frame(
    id     = c(1,1, 2,2, 3,3,3, 4,4),
    Tstart = c(0,0, 0,0, 0,0,1.6, 0,0),
    Tstop  = c(1.9,1.9, 2.2,2.2, 1.6,1.6,2.7, 0.4,0.4),
    from   = c(1,1, 1,1, 1,1,2, 1,1),
    to     = c(2,3, 2,3, 2,3,3, 2,3),
    trans  = c(1,2, 1,2, 1,2,3, 1,2),
    status = c(0,0, 0,1, 1,0,0, 1,0)
  )
}

test_that("state_at returns starting state at s = 0", {
  msd <- make_msd()
  out <- state_at(msd, s = 0.0)
  expect_equal(nrow(out), 4)
  expect_true(all(out$state == 1))
})

test_that("state_at identifies subjects who have transitioned to illness", {
  msd <- make_msd()
  out <- state_at(msd, s = 2.0)
  expect_equal(out$state[out$id == 3], 2)
})

test_that("state_at excludes absorbed subjects", {
  msd <- make_msd()
  out <- state_at(msd, s = 1.0)
  expect_false(4 %in% out$id)
})

test_that("state_at excludes subjects whose follow-up ended", {
  msd <- make_msd()
  out <- state_at(msd, s = 3.0)
  expect_false(2 %in% out$id)
})

test_that("state_at deduplicates competing-transition rows", {
  msd <- make_msd()
  out <- state_at(msd, s = 0.5)
  expect_equal(anyDuplicated(out$id), 0)
})

test_that("state_at validates input", {
  expect_error(state_at(data.frame(x = 1), s = 0), "missing columns")
  expect_error(state_at(make_msd(), s = c(1, 2)), "numeric scalar")
  expect_error(state_at(make_msd(), s = "a"), "numeric scalar")
})

#' Parse a patp Formula into Validated Interval Data
#'
#' Internal helper. Takes a formula of the form
#' \code{msm(Tstart, Tstop, Sstart, Sstop) ~ 1} or
#' \code{msm(Tstart, Tstop, Sstart, Sstop) ~ group_var}, plus a data
#' frame and ID column names, and returns a list with the assembled
#' interval data and an optional grouping vector.
#'
#' @param formula A formula. LHS must be a call to \code{msm()}; RHS
#'   must be either \code{1} (one-sample) or a single variable name
#'   (two-sample).
#' @param data A data frame to evaluate the formula in.
#' @param id Character. Name of the subject ID column in \code{data}.
#'   Required.
#' @param cluster Character or \code{NA}. Name of the cluster ID column
#'   in \code{data}. \code{NA} means no clustering (individual-level
#'   bootstrap).
#'
#' @return A list with elements:
#' \itemize{
#'   \item \code{data}: a data frame with columns \code{id}, \code{Tstart},
#'     \code{Tstop}, \code{Sstart}, \code{Sstop}, plus \code{cluster} if
#'     supplied and \code{group} if the formula has an RHS.
#'   \item \code{group}: \code{NULL} (one-sample) or a vector aligned with
#'     the rows of \code{data} (two-sample).
#'   \item \code{has_cluster}: logical, whether a cluster column was
#'     supplied.
#' }
#'
#' @keywords internal
parse_msm_formula <- function(formula, data, id, cluster = NA) {

  if (!inherits(formula, "formula")) {
    stop("'formula' must be a formula object")
  }
  if (!is.data.frame(data)) {
    stop("'data' must be a data frame")
  }
  if (missing(id) || is.null(id) || !is.character(id) || length(id) != 1L) {
    stop("'id' must be a single character string naming a column in 'data'")
  }
  if (!id %in% names(data)) {
    stop("id column '", id, "' not found in 'data'")
  }

  cluster_supplied <- !is.null(cluster) && !is.na(cluster)
  if (cluster_supplied) {
    if (!is.character(cluster) || length(cluster) != 1L) {
      stop("'cluster' must be a single character string or NA")
    }
    if (!cluster %in% names(data)) {
      stop("cluster column '", cluster, "' not found in 'data'")
    }
  }

  ## ---- Validate the LHS is a call to msm() ----
  lhs <- formula[[2L]]
  if (!is.call(lhs) || as.character(lhs[[1L]]) != "msm") {
    stop("formula's left-hand side must be a call to msm(), e.g. ",
         "msm(Tstart, Tstop, Sstart, Sstop) ~ 1")
  }
  if (length(lhs) != 5L) {  # msm + 4 args
    stop("msm() must be called with exactly 4 arguments: ",
         "Tstart, Tstop, Sstart, Sstop")
  }

  ## ---- Evaluate LHS in data ----
  msm_obj <- eval(lhs, envir = data, enclos = environment(formula))
  if (!inherits(msm_obj, "msm")) {
    stop("LHS evaluation did not produce an msm object")
  }
  if (nrow(msm_obj) != nrow(data)) {
    stop("msm object has ", nrow(msm_obj), " rows but 'data' has ",
         nrow(data), " rows")
  }

  ## ---- Examine RHS for grouping variable ----
  term_labels <- attr(stats::terms(formula), "term.labels")
  if (length(term_labels) > 1L) {
    stop("patp supports a single grouping variable on the right-hand ",
         "side; got ", length(term_labels))
  }

  group <- NULL
  if (length(term_labels) == 1L) {
    if (!term_labels %in% names(data)) {
      stop("grouping variable '", term_labels, "' not found in 'data'")
    }
    group <- data[[term_labels]]
    if (length(unique(stats::na.omit(group))) != 2L) {
      stop("grouping variable '", term_labels, "' must take exactly 2 ",
           "distinct non-NA values; got ",
           length(unique(stats::na.omit(group))))
    }
  }

  ## ---- Assemble output data frame ----
  out <- data.frame(
    id     = data[[id]],
    Tstart = msm_obj[, "Tstart"],
    Tstop  = msm_obj[, "Tstop"],
    Sstart = msm_obj[, "Sstart"],
    Sstop  = msm_obj[, "Sstop"]
  )

  if (cluster_supplied) out$cluster <- data[[cluster]]
  if (!is.null(group))  out$group   <- group

  list(
    data        = out,
    group       = group,
    has_cluster = cluster_supplied,
    group_name  = if (length(term_labels) == 1L) term_labels else NULL
  )
}

---
title: "Getting started with clusteredMSM"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Getting started with clusteredMSM}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 4
)
```

## Overview

`clusteredMSM` provides nonparametric analysis of clustered multistate
process data, based on Bakoyannis (2021) and Bakoyannis & Bandyopadhyay (2022). It exposes one user-facing
function, `patp()`, that:

* Estimates the population-averaged transition probability
  P(X(t) = j | X(s) = h) using the working-independence
  Aalen-Johansen estimator.
* Returns cluster-bootstrap pointwise confidence intervals and
  simultaneous bands.
* Conducts a two-sample Kolmogorov-Smirnov-type test when the formula
  has a grouping variable on the right-hand side.
* Supports a landmark variant for non-Markov processes.
* Natively handles non-monotone (recovery) multistate models.

The package depends only on `survival`.

## Input data format

`clusteredMSM` expects data in **interval format**: one row per
mutually-exclusive time interval per subject, with columns for
interval start time (`Tstart`), end time (`Tstop`), starting state
(`Sstart`), and ending state (`Sstop`). The column names are flexible
— they are passed by name through the formula.

Censoring is encoded as `Sstart == Sstop` on the **final row** of a
subject's record. Within each subject, intervals must be temporally
contiguous (`Tstop[k] == Tstart[k+1]`) and state contiguous
(`Sstop[k] == Sstart[k+1]`).

## A progressive illness-death example

```{r setup}
library(clusteredMSM)

# 1=Healthy, 2=Ill, 3=Dead. Allowed: 1->2, 1->3, 2->3.
tmat <- trans_mat(
  list(c(2, 3), 3, integer(0)),
  names = c("Healthy", "Ill", "Dead")
)
tmat
```

A small example dataset in interval format:

```{r data}
mydata <- data.frame(
  pid     = c(1, 1, 2, 3, 4, 4),
  site    = c(1, 1, 1, 2, 2, 2),
  treatment = c("A", "A", "A", "B", "B", "B"),
  t0      = c(0.0, 1.5, 0.0, 0.0, 0.0, 1.0),
  t1      = c(1.5, 3.0, 2.0, 1.0, 1.0, 2.5),
  s0      = c(1,   2,   1,   1,   1,   2),
  s1      = c(2,   3,   1,   3,   2,   3)
)
mydata
```

Subject 1: H -> I -> D. Subject 2: censored healthy at t = 2.0
(`s0 == s1`). Subject 3: died at t = 1.0. Subject 4: H -> I -> D.

## One-sample estimation

```{r estimate, eval = FALSE}
fit <- patp(
  msm(t0, t1, s0, s1) ~ 1,
  data    = mydata,
  tmat    = tmat,
  id      = "pid",
  cluster = "site",
  h = 1, j = 2, s = 0,
  B = 1000, cband = TRUE,
  seed = 1
)
fit
```

`fit$curves` contains the time-indexed point estimate of 
\begin{eqnarray*}
P(X(t) = j \,|\, X(s) = h) &=& P(X(t) = 2 \,|\, X(0) = 1) \\
&=& P(X(t) = 2),
\end{eqnarray*}
(second equality due to the fact that all start at state = 1 in the classical illness-death model), standard error, pointwise confidence interval, and (with `cband = TRUE`) simultaneous
band limits.

## Two-sample comparison (estimation + test in one call)

If the formula's right-hand side names a binary grouping variable,
`patp()` automatically estimates both curves and conducts a
Kolmogorov-Smirnov-type test:

```{r test, eval = FALSE}
tt <- patp(
  msm(t0, t1, s0, s1) ~ treatment,
  data    = mydata,
  tmat    = tmat,
  id      = "pid",
  cluster = "site",
  h = 1, j = 2, s = 0,
  B = 1000,
  seed = 1
)
tt
```

The `$curves` slot now has one row block per group. The `$test` slot
contains the observed K-S statistic and the bootstrap p-value.

## Recovery models (non-monotone processes)

For processes with cyclic transitions (e.g., illness-death with
recovery), use the long event format directly. The same `patp()` call
works:

```{r recovery, eval = FALSE}
tmat_rec <- trans_mat(
  list(c(2, 3), c(1, 3), integer(0)),
  names = c("Healthy", "Ill", "Dead")
)

# Subject who went Healthy -> Ill -> Healthy -> censored:
recovery_data <- data.frame(
  pid = c(1, 1, 1),
  t0  = c(0.0, 1.0, 2.0),
  t1  = c(1.0, 2.0, 3.5),
  s0  = c(1,   2,   1),
  s1  = c(2,   1,   1)    # last row: censored healthy
)

patp(msm(t0, t1, s0, s1) ~ 1,
     data = recovery_data, tmat = tmat_rec,
     id = "pid",
     h = 1, j = 2, s = 0,
     B = 500, seed = 1)
```

## Landmark variant

When `s > 0` and the Markov assumption is questionable, use
`LMAJ = TRUE`:

```{r lmaj, eval = FALSE}
patp(msm(t0, t1, s0, s1) ~ 1,
     data = mydata, tmat = tmat,
     id = "pid", cluster = "site",
     h = 1, j = 2, s = 1.0,
     LMAJ = TRUE, B = 1000, seed = 1)
```

## Inverse-cluster-size weighting

When cluster size is potentially informative (e.g., sicker centers contribute more patients), use the weighted estimator:

```{r weighted, eval = FALSE}
patp(msm(t0, t1, s0, s1) ~ 1,
     data = mydata, tmat = tmat,
     id = "pid", cluster = "site",
     h = 1, j = 2, s = 0,
     weighted = TRUE, B = 1000, seed = 1)
```

`weighted = TRUE` requires the `cluster` argument.

## What `B = 0` does

Setting `B = 0` returns the point estimate without any bootstrap
inference. This is permitted only for one-sample formulas; two-sample
formulas always require `B > 0` (since the entire point of a
two-sample analysis is the test). Use `B = 0` for fast exploratory
work, then switch to `B = 1000` for inference.

## References

Bakoyannis G (2021). Nonparametric analysis of nonhomogeneous
multistate processes with clustered observations. *Biometrics*
77(2):533-546. [doi:10.1111/biom.13327](https://doi.org/10.1111/biom.13327)

Bakoyannis G, Bandyopadhyay D (2022). Nonparametric tests for
multistate processes with clustered data. *Annals of the Institute
of Statistical Mathematics* 74(5):837-867.
[doi:10.1007/s10463-021-00819-x](https://doi.org/10.1007/s10463-021-00819-x)

Putter H, Spitoni C (2018). Non-parametric estimation of transition
probabilities in non-Markov multi-state models: the landmark
Aalen-Johansen estimator. *Statistical Methods in Medical Research*
27(7):2081-2092.
[doi:10.1177/0962280216674497](https://doi.org/10.1177/0962280216674497)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/trans_mat.R
\name{trans_mat}
\alias{trans_mat}
\title{Build a Transition Matrix for a Multistate Process}
\usage{
trans_mat(x, names = NULL)
}
\arguments{
\item{x}{A list of length K. Element \code{k} is an integer vector of
states reachable directly from state \code{k}. Use
\code{integer(0)} or \code{NULL} for absorbing states.}

\item{names}{Optional character vector of length K with state names.
If \code{NULL}, states are named \code{"1"}, \code{"2"}, ..., \code{"K"}.}
}
\value{
A K x K integer matrix. Entry \code{[h, j]} is the integer
  transition ID for the direct transition h -> j, or \code{NA} if no
  such transition exists. Transition IDs are assigned in row-major
  order (all transitions out of state 1 first, then state 2, etc.).
  The matrix has \code{dimnames} \code{list(from = names, to = names)}.
}
\description{
Constructs a square transition matrix in the format expected by
\code{\link{patp}} and \code{\link{prodint_AJ}}. It supports cyclic transitions
(e.g., illness-death with recovery).
}
\examples{
# Illness-death without recovery (progressive)
trans_mat(list(c(2, 3), 3, integer(0)),
          names = c("Healthy", "Ill", "Dead"))

# Illness-death WITH recovery (non-monotone)
trans_mat(list(c(2, 3), c(1, 3), integer(0)),
          names = c("Healthy", "Ill", "Dead"))

# Four-state multistate model with recovery from two intermediate states
trans_mat(list(c(2, 3, 4), c(1, 4), c(1, 4), integer(0)))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{step_interp}
\alias{step_interp}
\title{Interpolate a Step Function onto a Time Grid}
\usage{
step_interp(times, values, grid, init = 0)
}
\arguments{
\item{times}{Numeric. Jump times where the function changes.}

\item{values}{Numeric. Function values; \code{values[k]} is the
value on \code{[times[k], times[k+1])}.}

\item{grid}{Numeric. Output grid.}

\item{init}{Numeric scalar. Value to return for grid points before
\code{times[1]}. Default 0.}
}
\value{
Numeric vector of values on \code{grid}.
}
\description{
Right-continuous step interpolation of a (time, value) sequence onto
a target grid. Used to align bootstrap replicates onto a common time
grid so they can be assembled into a matrix.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{summary.patp}
\alias{summary.patp}
\title{Summary Method for patp Objects}
\usage{
\method{summary}{patp}(object, ...)
}
\arguments{
\item{object}{A \code{patp} object.}

\item{...}{Additional arguments (ignored).}
}
\value{
A list with named components for further inspection.
}
\description{
Returns the full curve(s) and (if applicable) the test result.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{add_cluster_sizes}
\alias{add_cluster_sizes}
\title{Add Cluster-Size Column for Inverse-Cluster-Size Weighting}
\usage{
add_cluster_sizes(data, cid, id)
}
\arguments{
\item{data}{A long-format data frame containing \code{cid} and \code{id}.}

\item{cid}{Character. Cluster ID column name.}

\item{id}{Character. Individual ID column name.}
}
\value{
The input data with an added \code{clust.size} column.
}
\description{
Computes the number of distinct individuals per cluster and merges
it back into the long-format data as a \code{clust.size} column.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cut_at_lm.R
\name{cut_at_lm}
\alias{cut_at_lm}
\title{Truncate a Long-Format Multistate Dataset at a Landmark Time}
\usage{
cut_at_lm(data, s)
}
\arguments{
\item{data}{A long-format data frame, typically produced by
\code{\link{intervals_to_long}()}, with columns \code{Tstart},
\code{Tstop}, \code{from}, \code{to}, \code{trans}, \code{status},
\code{id}, and (optionally) \code{cluster}.}

\item{s}{Numeric scalar. The landmark time.}
}
\value{
A data frame of the same shape as \code{data}, restricted and
  truncated as described above.
}
\description{
Restricts a long-format multistate dataset to follow-up after a
landmark time \code{s}. Intervals ending before \code{s} are removed;
intervals straddling \code{s} are truncated to start at \code{s}.
}
\details{
This function is used in the landmark Aalen-Johansen estimator, which
relaxes the Markov assumption by re-fitting hazards on the cohort
still under observation at time \code{s}.

Important: this function only truncates the time axis. To compute the
landmark estimator, you must also restrict to subjects who are in the
relevant starting state at time \code{s} -- use \code{state_at()} for
that step.

Intervals where \code{Tstop == s} exactly are dropped (the subject has
already transitioned by \code{s}); intervals where \code{Tstart >= s}
are kept unchanged; intervals straddling \code{s}
(\code{Tstart < s < Tstop}) have their \code{Tstart} reset to \code{s}.
If a straddling interval was going to end in an event
(\code{status == 1}) at \code{Tstop > s}, the event is preserved.
}
\examples{
data(example_msm)
tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
                  names = c("Healthy", "Ill", "Dead"))
long <- intervals_to_long(example_msm, tmat)

# Restrict follow-up to after the landmark time s = 1.5
long_lm <- cut_at_lm(long, s = 1.5)
head(long_lm)

}
\references{
Putter H, Spitoni C (2018). Non-parametric estimation of transition
probabilities in non-Markov multi-state models: the landmark
Aalen-Johansen estimator. \emph{Statistical Methods in Medical
Research} 27(7):2081-2092. \doi{10.1177/0962280216674497}

Bakoyannis G (2021). Nonparametric analysis of nonhomogeneous
multistate processes with clustered observations.
\emph{Biometrics} 77(2):533-546. \doi{10.1111/biom.13327}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate_intervals.R
\name{intervals_to_long}
\alias{intervals_to_long}
\title{Convert Validated Interval Data to Long Multistate Format}
\usage{
intervals_to_long(data, tmat)
}
\arguments{
\item{data}{A validated interval data frame with columns \code{id},
\code{Tstart}, \code{Tstop}, \code{Sstart}, \code{Sstop}, plus
optional \code{cluster} and \code{group} columns. Run
\code{\link{validate_intervals}()} first to confirm the data
satisfy the package's contiguity and absorbing-state rules.}

\item{tmat}{A K x K transition matrix from \code{\link{trans_mat}()}.}
}
\value{
A data frame in long multistate format with columns
  \code{id}, \code{Tstart}, \code{Tstop}, \code{from}, \code{to},
  \code{trans}, \code{status}, plus \code{cluster} and \code{group}
  if present in \code{data}.
}
\description{
Expands each row of an interval-format multistate dataset into one
row per allowed transition out of the interval's starting state,
with \code{status = 1} on the row whose destination matches the
actual transition (and \code{0} on the others). The result is the
long format expected by \code{\link{fit_chaz}} and other
counting-process style routines.
}
\details{
Most users do not need to call this function directly --
\code{\link{patp}()} runs the full pipeline (parse, validate,
expand, fit). It is exported so advanced users can compose the
lower-level building blocks (\code{\link{fit_chaz}},
\code{\link{prodint_AJ}}, \code{\link{cluster_boot}}) into custom
analyses.
}
\examples{
data(example_msm)
tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
                  names = c("Healthy", "Ill", "Dead"))
long <- intervals_to_long(example_msm, tmat)
head(long)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/methods.R
\name{print.patp}
\alias{print.patp}
\title{Print Method for patp Objects}
\usage{
\method{print}{patp}(x, digits = max(3L, getOption("digits") - 3L), ...)
}
\arguments{
\item{x}{A \code{patp} object.}

\item{digits}{Integer. Number of significant digits for printing.}

\item{...}{Additional arguments (ignored).}
}
\value{
The object \code{x}, invisibly.
}
\description{
Concise summary of a \code{\link{patp}} fit. For two-sample fits,
includes the K-S statistic and p-value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_clusters}
\alias{check_clusters}
\title{Validate Independent-Clusters Assumption}
\usage{
check_clusters(data, cid, id)
}
\arguments{
\item{data}{A data frame.}

\item{cid}{Character. Cluster ID column name.}

\item{id}{Character. Individual ID column name.}
}
\description{
Checks that no individual ID appears in more than one cluster.
Stops with an informative error if the assumption is violated.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validate_intervals.R
\name{validate_intervals}
\alias{validate_intervals}
\title{Strictly Validate Multistate Data in Interval Format}
\usage{
validate_intervals(data, tmat, tol = 1e-09)
}
\arguments{
\item{data}{A data frame with columns \code{id}, \code{Tstart},
\code{Tstop}, \code{Sstart}, \code{Sstop}.}

\item{tmat}{A K x K transition matrix from \code{\link{trans_mat}()}.}

\item{tol}{Numeric tolerance for temporal-contiguity checks.
Default 1e-9.}
}
\value{
Invisibly returns \code{TRUE} if all checks pass. Stops with
  an error otherwise.
}
\description{
Performs strict validation on a data frame of multistate data in interval
format (the output of \code{\link{parse_msm_formula}()}). Stops with
an informative error on the first violation.
}
\details{
Validation rules, applied in order:
\enumerate{
  \item Required columns are present and of correct type.
  \item Every row has \code{Tstart < Tstop}.
  \item All states are integers in \code{1..K}.
  \item Within each subject, rows are sorted by time.
  \item Within each subject, rows are temporally contiguous:
    \code{Tstop[k] == Tstart[k+1]}.
  \item Within each subject, rows are spatially contiguous:
    \code{Sstop[k] == Sstart[k+1]}.
  \item No row has \code{Sstart} equal to an absorbing state.
  \item No row follows one whose \code{Sstop} is absorbing.
  \item Each non-censored transition (\code{Sstart != Sstop}) is
    allowed by \code{tmat}.
  \item \code{Sstart == Sstop} (censoring) is permitted only on the
    final row of a subject's record.
}
}
\examples{
data(example_msm)
tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
                  names = c("Healthy", "Ill", "Dead"))
validate_intervals(example_msm, tmat)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prodint_AJ.R
\name{prodint_AJ}
\alias{prodint_AJ}
\title{Aalen-Johansen Product Integral for General Multistate Processes}
\usage{
prodint_AJ(haz_df, tmat, predt = 0, h = 1)
}
\arguments{
\item{haz_df}{Data frame with columns \code{time}, \code{Haz},
\code{trans}, typically the output of \code{fit_chaz()} or the
\code{$Haz} element of an \code{mstate::msfit} object. \code{Haz}
is the cumulative (not incremental) hazard.}

\item{tmat}{A K x K transition matrix from \code{trans_mat()}.
Cyclic transitions (both \code{[h, j]} and \code{[j, h]} non-NA)
are permitted.}

\item{predt}{Numeric scalar. Starting time s. Probabilities are
computed for t > predt. Default 0.}

\item{h}{Integer in 1..K. Starting state.}
}
\value{
A data frame with columns \code{time}, \code{pstate1}, ...,
  \code{pstateK}. Row k gives \eqn{P(predt, time_k)[h, ]}, the
  probabilities of being in each state at \code{time_k} given the
  process was in state \code{h} at \code{predt}.
}
\description{
Computes the row of the transition probability matrix
\deqn{P(s, t) = \prod_{u \in (s, t]} (I + dA(u))}
starting from state \code{h}, given Nelson-Aalen
transition hazard estimates.
}
\details{
At each unique jump time \eqn{u}, the increment matrix
\eqn{dA(u)} has off-diagonal entry \eqn{(h, j)} equal to the
Nelson-Aalen increment of the cumulative hazard for transition
\eqn{h \to j}, and diagonal entries equal to the negative row sums.
The transition probability matrix is then updated by
\eqn{P \leftarrow P (I + dA(u))}.

Computational complexity is O(J K^2) where J is the number of unique
jump times and K is the number of states.
}
\examples{
data(example_msm)
tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
                  names = c("Healthy", "Ill", "Dead"))

long <- intervals_to_long(example_msm, tmat)
haz  <- fit_chaz(long, tmat)
P    <- prodint_AJ(haz, tmat, predt = 0, h = 1)
head(P)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit_chaz.R
\name{fit_chaz}
\alias{fit_chaz}
\title{Fit Cox Model Stratified by Transition and Return Cumulative Hazards}
\usage{
fit_chaz(data, tmat, weights = NULL)
}
\arguments{
\item{data}{A data frame in long multistate format with columns
\code{Tstart}, \code{Tstop}, \code{status}, and \code{trans}.
Typically the output of \code{\link{intervals_to_long}()}.}

\item{tmat}{A K x K transition matrix from \code{trans_mat()}. Used
only to validate that all transition IDs in \code{data} are defined
in \code{tmat}; not used in the fit itself.}

\item{weights}{Optional numeric vector of length \code{nrow(data)}
with observation weights. Pass \code{1 / data$clust.size} for the
inverse-cluster-size-weighted estimator. \code{NULL} (default)
means unweighted.}
}
\value{
A data frame with columns:
\itemize{
  \item \code{time}: jump time of the transition.
  \item \code{Haz}: Breslow estimate of the cumulative transition
    hazard at \code{time}.
  \item \code{trans}: integer transition ID (matches \code{tmat}).
}
Sorted by \code{trans} then \code{time}.
}
\description{
Fits a Cox proportional hazards model stratified by transition type and
returns the Breslow-estimated cumulative transition hazards in long
format.
}
\details{
The model fitted is
\code{coxph(Surv(Tstart, Tstop, status) ~ strata(trans),
method = "breslow")}.
This is the Andersen-Gill counting-process formulation, which handles
multiple intervals per subject -- including non-monotone processes
with recovery -- provided the input data was correctly constructed
(one row per sojourn x candidate transition).
}
\examples{
data(example_msm)
tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
                  names = c("Healthy", "Ill", "Dead"))

# fit_chaz takes long-format data; intervals_to_long() converts
# the interval format users supply. Most users reach fit_chaz
# indirectly via patp() -- this example shows the manual pipeline.
long <- intervals_to_long(example_msm, tmat)
haz  <- fit_chaz(long, tmat)
head(haz)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{example_msm}
\alias{example_msm}
\title{Synthetic Clustered Illness-Death-with-Recovery Data}
\format{
A data frame with 81 rows (one row per mutually-exclusive
  time interval per subject) and 7 columns:
\describe{
  \item{id}{Integer subject identifier (40 subjects, ids 1..40).}
  \item{cluster}{Integer cluster identifier (8 clusters, 5 subjects
    each).}
  \item{treatment}{Binary treatment indicator (0 or 1), balanced
    20/20 across subjects. Treatment 1 has a higher illness hazard
    than treatment 0.}
  \item{Tstart}{Numeric start time of the interval.}
  \item{Tstop}{Numeric end time of the interval.}
  \item{Sstart}{Integer state during the interval: 1 = Healthy,
    2 = Ill, 3 = Dead (absorbing).}
  \item{Sstop}{Integer state at \code{Tstop}, or equal to
    \code{Sstart} on a final, censored row.}
}
}
\source{
Simulated. See \code{data-raw/example_msm.R} in the package
  sources.
}
\usage{
example_msm
}
\description{
A synthetic interval-format multistate dataset used in package
examples, the README, and the vignette. The data are generated from
an illness-death-with-recovery process with cluster-level frailty,
so they exercise the package's headline feature: support for
non-monotone (cyclic) transitions.
}
\details{
The transition structure is
\code{trans_mat(list(c(2, 3), c(1, 3), integer(0)))}, i.e.
Healthy <-> Ill, Healthy -> Dead, Ill -> Dead. Within each subject
rows are temporally and spatially contiguous; censoring is encoded
as \code{Sstart == Sstop} on the final row.

The same data are also shipped as a CSV at
\code{system.file("extdata", "example_data.csv", package = "clusteredMSM")}
for users who prefer to mimic loading their own file.

The generation script lives in \code{data-raw/example_msm.R} and is
fully reproducible (\code{set.seed(2026)}).
}
\examples{
data(example_msm)
head(example_msm)
table(example_msm$Sstart, example_msm$Sstop)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference.R
\name{confidence_band}
\alias{confidence_band}
\title{Simultaneous Confidence Band via Cluster-Bootstrap Quantile}
\usage{
confidence_band(point, boot, times, level = 0.95, trim = c(0.05, 0.95))
}
\arguments{
\item{point}{Numeric vector of point estimates over a time grid.}

\item{boot}{Numeric matrix of bootstrap replicates of \code{point}
on the original probability scale: rows correspond to time
points (matching \code{point}), columns to bootstrap replicates.}

\item{times}{Numeric vector of times matching the rows of \code{boot}
and the entries of \code{point}.}

\item{level}{Confidence level. Default 0.95.}

\item{trim}{Numeric vector of length 2. Lower and upper quantiles of
the jump-time distribution at which to clip the band (avoids the
"fans out" behavior near the extremes of follow-up). Default
\code{c(0.05, 0.95)}.}
}
\value{
A list with elements \code{ll.band} and \code{ul.band}:
  numeric vectors of band limits, with \code{NA} outside the trimmed
  range or where \code{point} is at the boundary 0/1.
}
\description{
Constructs a simultaneous (uniform-over-time) confidence band for a
transition probability curve. The band is built on the cloglog
scale: for each bootstrap replicate, the standardized supremum
\eqn{\sup_t |g(\hat P_b(t)) - g(\hat P(t))| / \mathrm{SE}_g(t)} is
computed, where \eqn{\mathrm{SE}_g(t)} is the delta-method SE on
the cloglog scale. The \code{level} quantile of those suprema is
the critical value, and the band is back-transformed to the
probability scale.
}
\details{
Only one bootstrap on the original (probability) scale is required:
\eqn{\mathrm{SE}_g(t)} is obtained by the delta method from
\eqn{\mathrm{SE}(\hat P(t)) = \mathrm{sd}(\hat P^*(t))}, and the
cloglog values \eqn{g(\hat P_b(t))} are computed by transforming
the existing replicates pointwise -- no separate cloglog-scale
bootstrap is run.
}
\section{Note}{

This version uses an equal-precision-type band on the cloglog
scale, in which the bootstrap deviation is standardized pointwise
by the delta-method standard error of \eqn{P(t)}. By the self-
cancellation of \eqn{\sqrt{n}} factors, this is asymptotically
equivalent to standardizing the limit process by its asymptotic
standard error on the cloglog scale, which is the standard
equal-precision construction. This construction is asymptotically
valid under conditions C1-C6 of Bakoyannis (2021), but differs
from the Hall-Wellner-type weight proposed in Section 2.3 of that
paper. Both constructions yield asymptotically valid simultaneous
bands. The paper's Hall-Wellner-type construction will be added as
a \code{band_method} option in a future release and will become
the default.
}

\references{
Bakoyannis, G. (2021). Nonparametric analysis of nonhomogeneous
multistate processes with clustered observations.
\emph{Biometrics}, 77(2), 533-546. \doi{10.1111/biom.13327}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parse_formula.R
\name{parse_msm_formula}
\alias{parse_msm_formula}
\title{Parse a patp Formula into Validated Interval Data}
\usage{
parse_msm_formula(formula, data, id, cluster = NA)
}
\arguments{
\item{formula}{A formula. LHS must be a call to \code{msm()}; RHS
must be either \code{1} (one-sample) or a single variable name
(two-sample).}

\item{data}{A data frame to evaluate the formula in.}

\item{id}{Character. Name of the subject ID column in \code{data}.
Required.}

\item{cluster}{Character or \code{NA}. Name of the cluster ID column
in \code{data}. \code{NA} means no clustering (individual-level
bootstrap).}
}
\value{
A list with elements:
\itemize{
  \item \code{data}: a data frame with columns \code{id}, \code{Tstart},
    \code{Tstop}, \code{Sstart}, \code{Sstop}, plus \code{cluster} if
    supplied and \code{group} if the formula has an RHS.
  \item \code{group}: \code{NULL} (one-sample) or a vector aligned with
    the rows of \code{data} (two-sample).
  \item \code{has_cluster}: logical, whether a cluster column was
    supplied.
}
}
\description{
Internal helper. Takes a formula of the form
\code{msm(Tstart, Tstop, Sstart, Sstop) ~ 1} or
\code{msm(Tstart, Tstop, Sstart, Sstop) ~ group_var}, plus a data
frame and ID column names, and returns a list with the assembled
interval data and an optional grouping vector.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cluster_boot.R
\name{cluster_boot}
\alias{cluster_boot}
\title{Generic Cluster Bootstrap Engine}
\usage{
cluster_boot(
  data,
  cluster,
  B,
  fn,
  ...,
  strata = NULL,
  seed = NULL,
  verbose = FALSE
)
}
\arguments{
\item{data}{A data frame containing the cluster ID column named by
\code{cluster}. All other columns are passed through unchanged to
\code{fn}.}

\item{cluster}{Character. Name of the cluster ID column.}

\item{B}{Integer. Number of bootstrap replications.}

\item{fn}{Function. Called as \code{fn(boot_data, ...)} once per
bootstrap replicate. Must return a numeric vector of fixed length
(the length must be the same across replicates -- typically achieved
by evaluating the statistic on a fixed grid of times).}

\item{...}{Additional arguments passed to \code{fn}.}

\item{strata}{Optional named vector mapping each unique cluster ID
(as character) to a stratum label. When supplied, resampling is
stratified: in each replicate, the original number of clusters in
each stratum is drawn with replacement from the clusters in that
stratum (so per-stratum cluster counts are preserved across
replicates). When \code{NULL} (default), unstratified resampling
is used.}

\item{seed}{Optional integer. If non-NULL, \code{set.seed(seed)} is
called before bootstrapping for reproducibility.}

\item{verbose}{Logical. If \code{TRUE}, prints a progress message
every 100 replicates. Default \code{FALSE}.}
}
\value{
A numeric matrix with \code{length(fn(data, ...))} rows and
  \code{B} columns. Column \code{b} is the result of \code{fn} applied
  to bootstrap replicate \code{b}.
}
\description{
Performs the nonparametric cluster bootstrap: resamples whole clusters
with replacement and applies a user-supplied statistic function to
each bootstrap replicate.
}
\details{
The bootstrap is nonparametric and at the cluster level: in each
replicate, n clusters are sampled with replacement (where n is the
number of unique values of \code{data[[cluster]]}), and the resulting
cluster-bound rows are stacked into a new data frame.

Resampled clusters are re-IDed (1, 2, ..., n) so that downstream
code treating cluster IDs as distinct labels (e.g., \code{tapply}
aggregations) works correctly even when the same original cluster
appears multiple times in a replicate.

Stratified resampling (\code{strata}) supports cluster-randomized
designs in which each cluster belongs to a single group (case ii of
Bakoyannis & Bandyopadhyay 2022): the per-stratum cluster counts
are fixed across replicates, so a replicate can never wipe out one
of the groups.

The bootstrap is statistic-agnostic: the same engine is used for
point-estimator standard errors, two-sample tests, and confidence
bands. Whatever \code{fn} returns is what gets bootstrapped.
}
\examples{
data(example_msm)

# Cluster-bootstrap a simple summary: the mean of Tstop across
# rows. The user-supplied fn can return any fixed-length numeric
# vector; the typical use case is a transition probability curve
# evaluated on a fixed time grid (which is exactly what patp() does
# under the hood).
boot <- cluster_boot(
  data = example_msm, cluster = "cluster", B = 50,
  fn   = function(d) mean(d$Tstop),
  seed = 1
)
c(point = mean(boot, na.rm = TRUE),
  se    = stats::sd(boot, na.rm = TRUE))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/state_at.R
\name{state_at}
\alias{state_at}
\title{Identify Each Subject's State at a Given Time}
\usage{
state_at(data, s, id = "id")
}
\arguments{
\item{data}{A data frame in long format with columns \code{Tstart},
\code{Tstop}, \code{from}, \code{to}, \code{status}, and the subject
ID column named by \code{id}.}

\item{s}{Numeric scalar. The time at which to evaluate states.}

\item{id}{Character. Name of the subject ID column. Default \code{"id"}.}
}
\value{
A data frame with columns \code{<id>} and \code{state}, one row
  per subject who is under observation at time \code{s}. Subjects whose
  follow-up has not yet started (\code{min(Tstart) > s}) or who have
  already entered an absorbing state (\code{max(Tstop) <= s} with
  \code{status == 1}) are excluded.
}
\description{
Given a long-format multistate dataset, returns the state each subject
occupies at time \code{s}. Used by the landmark Aalen-Johansen estimator
to identify the risk set at a conditioning time.
}
\details{
A subject's state at time \code{s} is the \code{from} state of the
interval that contains \code{s} (i.e., \code{Tstart <= s < Tstop}).
Because the long format duplicates intervals across competing
transitions, the result is deduplicated to one row per subject.

For exact ties (\code{s == Tstop}), the subject is treated as having
transitioned: they appear in their new state if a subsequent interval
exists, otherwise they are excluded as absorbed/censored.
}
\examples{
data(example_msm)
tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
                  names = c("Healthy", "Ill", "Dead"))
long <- intervals_to_long(example_msm, tmat)

# Each subject's state at the landmark time s = 1.5
head(state_at(long, s = 1.5, id = "id"))

}
\references{
Putter H, Spitoni C (2018). Non-parametric estimation of transition
probabilities in non-Markov multi-state models: the landmark
Aalen-Johansen estimator. \emph{Statistical Methods in Medical
Research} 27(7):2081-2092. \doi{10.1177/0962280216674497}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/msm.R
\name{msm}
\alias{msm}
\title{Construct a Multistate Interval Object for Use in patp() Formulas}
\usage{
msm(Tstart, Tstop, Sstart, Sstop)
}
\arguments{
\item{Tstart}{Numeric vector. Start time of each interval.}

\item{Tstop}{Numeric vector. End time of each interval.}

\item{Sstart}{Integer-valued numeric vector. State occupied at
\code{Tstart}. States must be whole numbers; the matrix backing
stores them as double (matrices have a single storage mode), but
values are validated to be integer-valued.}

\item{Sstop}{Integer-valued numeric vector. State occupied at
\code{Tstop}. Same storage caveat as \code{Sstart}. If
\code{Sstart == Sstop}, the row represents a censored interval;
this is permitted only on the final row of a subject's record.}
}
\value{
A four-column matrix of class \code{"msm"} with columns
  \code{Tstart}, \code{Tstop}, \code{Sstart}, \code{Sstop}.
}
\description{
Packages four columns -- interval start time, interval end time,
starting state, and ending state -- into a single object suitable
for use as the response in a \code{\link{patp}} formula.
Conceptually analogous to \code{\link[survival]{Surv}()} in survival
analysis: the user calls it inside a formula
(\code{msm(Tstart, Tstop, Sstart, Sstop) ~ ...}).
}
\details{
Used inside a model formula passed to \code{\link{patp}}, e.g.
\code{patp(msm(Tstart, Tstop, Sstart, Sstop) ~ 1, ...)}. The four
arguments can be named anything in the user's data -- the formula
machinery looks them up in the supplied \code{data}.
}
\examples{
Tstart <- c(0,   1.5, 0,   0)
Tstop  <- c(1.5, 3.0, 2.0, 1.2)
Sstart <- c(1,   2,   1,   1)
Sstop  <- c(2,   3,   3,   1)   # last row censored (S unchanged)
obj <- msm(Tstart, Tstop, Sstart, Sstop)
head(obj)

}
\seealso{
\code{\link{patp}}, \code{\link{validate_intervals}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference.R
\name{ks_pvalue}
\alias{ks_pvalue}
\title{Two-Sample Kolmogorov-Smirnov-Type Test via Cluster Bootstrap}
\usage{
ks_pvalue(diff_point, diff_boot, scale)
}
\arguments{
\item{diff_point}{Numeric vector. Observed group difference of the
point estimator (\eqn{\hat P_1 - \hat P_0}) on a fixed time grid.}

\item{diff_boot}{Numeric matrix. Cluster bootstrap replicates of the
group difference: rows = time points (matching \code{diff_point}),
columns = replicates.}

\item{scale}{Numeric scalar. Asymptotic scaling factor applied to
both the observed sup-statistic and the bootstrap analogue.
Use \eqn{\sqrt{n}} (\eqn{n} = total clusters) for the
dependent-groups design (Bakoyannis 2021, Theorem 3); use
\eqn{\sqrt{n_1 n_2 / (n_1 + n_2)}} for the cluster-randomized
design (Bakoyannis & Bandyopadhyay 2022, Theorem 2).}
}
\value{
A list with elements \code{statistic} (the observed
  sup-statistic, on the chosen scale) and \code{p.value}.
}
\description{
Computes a p-value for the null hypothesis that a transition
probability curve is equal across two groups, based on the supremum
of the absolute difference between group-specific Aalen-Johansen
estimators and a cluster-bootstrap reference distribution.
}
\details{
The test statistic is \eqn{T = c_n \sup_t | \hat P_1(t) -
\hat P_0(t) |} with \eqn{c_n =} \code{scale}, and the null
distribution is approximated by the bootstrap analogue applied to
centered bootstrap differences. The empirical p-value is invariant
to \code{scale} (it appears on both sides of the comparison); the
role of \code{scale} is to put the reported statistic on the
correct asymptotic scale per the relevant theorem.

This v0.1 implementation uses a unit weight: every time point on
the grid contributes equally to the supremum. Bakoyannis (2021)
Section 2.5 instead recommends a time-varying weight
\eqn{W(t) = \prod_p Y_p(t) / \sum_p Y_p(t)} (the harmonic mean of
the per-group at-risk processes), which downweights regions where
one group's at-risk set is small and naturally tames tail
instability of the difference estimator. A weighted variant
following Bakoyannis (2021) Section 2.5 and Bakoyannis & Bandyopadhyay
(2022) is planned for v0.2.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patp.R
\name{patp}
\alias{patp}
\title{Population-Averaged Transition Probabilities for Multistate Process Data}
\usage{
patp(
  formula,
  data,
  tmat,
  id,
  cluster = NA,
  h,
  j,
  s = 0,
  weighted = FALSE,
  LMAJ = FALSE,
  B = 1000,
  cband = FALSE,
  design = c("auto", "shared", "cluster_random", "indep_random"),
  level = 0.95,
  seed = NULL
)
}
\arguments{
\item{formula}{A formula of the form
\code{msm(Tstart, Tstop, Sstart, Sstop) ~ 1} for a one-sample
analysis, or \code{msm(Tstart, Tstop, Sstart, Sstop) ~ group} for
a two-sample analysis (estimate + test).}

\item{data}{A data frame containing the variables in \code{formula}
plus the \code{id} (and optional \code{cluster}) columns.}

\item{tmat}{A K x K transition matrix, typically built with
\code{\link{trans_mat}()}.}

\item{id}{Character. Name of the subject ID column. Required.}

\item{cluster}{Character or \code{NA}. Name of the cluster ID column.
When \code{NA} (default), the bootstrap resamples individuals;
when supplied, it resamples whole clusters.}

\item{h}{Integer in 1..K. Starting state.}

\item{j}{Integer in 1..K. Ending state.}

\item{s}{Numeric scalar. Conditioning time. Default 0.}

\item{weighted}{Logical. Inverse-cluster-size weighting to account for 
potentially informative cluster size (requires \code{cluster}). 
Default \code{FALSE}.}

\item{LMAJ}{Logical. Use the landmark Aalen-Johansen estimator
(recommended when \code{s > 0} and the Markov assumption is
questionable). Default \code{FALSE}.}

\item{B}{Integer. Number of bootstrap replications. \code{B = 0}
skips inference and is permitted only for one-sample formulas.
Default 1000.}

\item{cband}{Logical. If \code{TRUE}, return simultaneous
confidence band limits at the level set by \code{level}
(recommended \code{B >= 1000}).}

\item{design}{Character, two-sample only. One of
\code{"auto"} (default), \code{"shared"},
\code{"cluster_random"}, \code{"indep_random"}. Selects the
two-sample regime; see the \emph{Two-sample designs} section.
\code{"auto"} infers the regime from the cluster/group
structure of the data, defaulting to \code{"indep_random"} when
each cluster carries a single group (a safer default than
assuming cluster randomization). When that fallback fires, a warning
is emitted asking the user to set \code{"cluster_random"}
explicitly if the per-group cluster counts were fixed by cluster
randomization.}

\item{level}{Confidence level. Default 0.95.}

\item{seed}{Optional integer for reproducible bootstrap.}
}
\value{
An S3 object of class \code{patp} containing:
\itemize{
  \item \code{call}, \code{formula}: the original call and formula.
  \item \code{curves}: a data frame with columns \code{time},
    \code{P}, \code{group} (if two-sample), and (if \code{B > 0})
    \code{se}, \code{ll}, \code{ul} -- and \code{ll.band},
    \code{ul.band} if \code{cband = TRUE}.
  \item \code{test}: \code{NULL} (one-sample) or a list with the
    observed K-S statistic and bootstrap p-value (two-sample).
  \item \code{n_subjects}, \code{n_clusters}, \code{groups},
    \code{h}, \code{j}, \code{s}, \code{B}.
}
}
\description{
Computes the working-independence Aalen-Johansen estimator of the
transition probability \eqn{P(X(t) = j \mid X(s) = h)} for clustered or
independent multistate process data, with cluster-bootstrap
standard errors, pointwise confidence intervals, and (optionally)
simultaneous confidence bands. If a grouping variable is supplied
on the right-hand side of the formula, also conducts a two-sample
Kolmogorov-Smirnov-type test of curve equality.
}
\section{Two-sample designs}{

Three regimes are supported, each with its own bootstrap scheme
and asymptotic scaling factor; the test statistic in every case is
\eqn{T = c_n \sup_t |\hat P_1(t) - \hat P_0(t)|}.
\describe{
  \item{\code{"shared"} (case i): dependent groups (multicenter
    trial).}{Every cluster contributes observations from both
    groups -- e.g., each hospital randomizes its patients to
    treatment or control. Unstratified cluster bootstrap;
    \eqn{c_n = \sqrt{n}} (Bakoyannis 2021, Theorem 3), with
    \eqn{n} the total number of clusters.}
  \item{\code{"cluster_random"} (case ii.a): cluster-randomized
    trial.}{Each cluster is assigned to exactly one group, with
    per-group cluster counts \eqn{n_1, n_2} fixed by the
    randomization. Stratified cluster bootstrap (resample within
    each group); \eqn{c_n = \sqrt{n_1 n_2 / (n_1 + n_2)}}
    (Bakoyannis & Bandyopadhyay 2022, Theorem 2). This regime
    must be opted into; \code{"auto"} will not select it.}
  \item{\code{"indep_random"} (case ii.b): independent
    observational comparison.}{Each cluster carries one group,
    but \eqn{n_1, n_2} are random (the population, not the
    analyst, decided who landed in which group). Unstratified
    cluster bootstrap; \eqn{c_n = \sqrt{n_1 n_2 / (n_1 + n_2)}}.
    The asymptotic regime is the same two-independent-samples
    limit as \code{"cluster_random"}; only the bootstrap
    differs. This is the \code{"auto"} default when each cluster
    carries a single group.}
}
Mixed structures (some clusters carry both groups, some only one)
are not supported in this version.

\code{patp()} validates the supplied data against the requested
design and stops with an informative error if they disagree.

The K-S statistic in v0.1 uses unit weight at every time point.
Weighted variants -- following the harmonic-mean weight
\eqn{W(t) = \prod_p Y_p(t) / \sum_p Y_p(t)} of Bakoyannis (2021)
Section 2.5 and the related construction in Bakoyannis & Bandyopadhyay
(2022) -- are planned for v0.2.
}

\section{Standard errors and confidence intervals}{

The cluster bootstrap is run \emph{once}, on the original
probability scale, in line with Bakoyannis (2021) Theorem 2. The
\code{se} column in \code{curves} is the bootstrap standard
deviation of \eqn{\hat P(t)} on that scale, i.e.
\code{apply(boot_matrix, 1, sd, na.rm = TRUE)}.

Pointwise CIs (\code{ll}, \code{ul}) are then built on the cloglog
scale \eqn{g(p) = \log(-\log p)} using the delta-method
standardization \eqn{\mathrm{SE}_g(t) = \mathrm{SE}(\hat P(t)) /
|\hat P(t) \log \hat P(t)|}, and back-transformed via
\eqn{p = \exp(-\exp(\cdot))}. Simultaneous bands
(\code{ll.band}, \code{ul.band}) use the same \eqn{\mathrm{SE}_g};
the \code{level}-quantile of the supremum gives the critical value.

Because the cloglog transformation is nonlinear, \code{se} and the
resulting CI widths are \emph{not} equal in general -- the
CI is asymmetric on the probability scale. Report \code{se} for
descriptive purposes; use \code{ll}/\code{ul} for inference.

The simultaneous band is trimmed to the central 90\% of observed
jump times by default to avoid the band fanning out near the
extremes of follow-up; outside that window \code{ll.band} and
\code{ul.band} are \code{NA}. See \code{\link{confidence_band}()}
for the \code{trim} argument.

Note: in this version the simultaneous band uses an
equal-precision-type construction on the cloglog scale, which is
asymptotically valid but differs from the Hall-Wellner-type
construction of Bakoyannis (2021) Section 2.3 (planned for a
future release). See \code{\link{confidence_band}()} for the full
disclosure.
}

\examples{
data(example_msm)
tmat <- trans_mat(list(c(2, 3), c(1, 3), integer(0)),
                  names = c("Healthy", "Ill", "Dead"))

# One-sample: P(Ill at t | Healthy at 0). B is small here for speed;
# use B >= 1000 for reported results.
fit <- patp(msm(Tstart, Tstop, Sstart, Sstop) ~ 1,
            data = example_msm, tmat = tmat,
            id = "id", cluster = "cluster",
            h = 1, j = 2, s = 0,
            B = 50, seed = 1)
fit

# Two-sample (estimate + test in one call). Each cluster in
# example_msm carries both treatment levels, so design = "shared".
tt <- patp(msm(Tstart, Tstop, Sstart, Sstop) ~ treatment,
           data = example_msm, tmat = tmat,
           id = "id", cluster = "cluster",
           h = 1, j = 2, B = 50,
           design = "shared", seed = 1)
tt

}
\references{
Bakoyannis, G. (2021). Nonparametric analysis of nonhomogeneous
multistate processes with clustered observations.
\emph{Biometrics}, 77(2), 533-546. \doi{10.1111/biom.13327}

Bakoyannis, G., & Bandyopadhyay, D. (2022). Nonparametric tests
for multistate processes with clustered data. \emph{Annals of the
Institute of Statistical Mathematics}, 74(5), 837-867.
\doi{10.1007/s10463-021-00819-x}

Putter H, Spitoni C (2018). Non-parametric estimation of transition
probabilities in non-Markov multi-state models: the landmark
Aalen-Johansen estimator. \emph{Statistical Methods in Medical
Research} 27(7):2081-2092. \doi{10.1177/0962280216674497}
}
\seealso{
\code{\link{msm}}, \code{\link{trans_mat}},
  \code{\link{validate_intervals}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/inference.R
\name{ci_cloglog}
\alias{ci_cloglog}
\title{Pointwise Confidence Intervals via Complementary Log-Log Transformation}
\usage{
ci_cloglog(point, se, level = 0.95)
}
\arguments{
\item{point}{Numeric vector of point estimates in (0, 1), length T.}

\item{se}{Numeric vector of bootstrap standard errors of \code{point}
on the probability scale, same length as \code{point} (i.e.
\code{apply(boot_matrix, 1, sd, na.rm = TRUE)}).}

\item{level}{Confidence level. Default 0.95.}
}
\value{
A list with elements \code{ll} and \code{ul}: numeric vectors
  of lower and upper confidence limits, the same length as
  \code{point}.
}
\description{
Computes pointwise confidence intervals for a transition probability
estimate using the cloglog transformation \eqn{g(p) = \log(-\log p)}
with a delta-method standard error scaling. The bootstrap is run on the 
original probability scale; the cloglog SE is then derived analytically.
}
\details{
By the delta method,
\eqn{\mathrm{SE}(g(\hat P)) = \mathrm{SE}(\hat P) / |\hat P \log \hat P|}
(here \eqn{|g'(p)| = 1/|p \log p|} on (0,1)). The interval is built
symmetrically on the cloglog scale and back-transformed via
\eqn{p = \exp(-\exp(\cdot))}. Because \eqn{g} is monotone
decreasing on (0, 1), the upper end on the cloglog scale becomes
the \emph{lower} end on the probability scale, and vice versa.

For point estimates exactly equal to 0 or 1 the cloglog is
undefined and \code{NA} is returned at those positions.
}
